# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six
import time

from ..const import ORDER_STATUS, SIDE, POSITION_EFFECT, ORDER_TYPE
from ..utils import id_gen
from ..utils.repr import property_repr, properties
from ..utils.logger import user_system_log


OrderPersistMap = {
    "_order_id": "_order_id",
    "_calendar_dt": "_calendar_dt",
    "_trading_dt": "_trading_dt",
    "_quantity": "_quantity",
    "_order_book_id": "_order_book_id",
    "_side": "_side",
    "_position_effect": "_position_effect",
    "_message": "_message",
    "_filled_quantity": "_filled_quantity",
    "_status": "_status",
    "_frozen_price": "_frozen_price",
    "_type": "_type",
    "_avg_price": "_avg_price",
    "_transaction_cost": "_transaction_cost",
}


class Order(object):

    order_id_gen = id_gen(int(time.time()))

    __repr__ = property_repr

    def __init__(self):
        self._order_id = None
        self._calendar_dt = None
        self._trading_dt = None
        self._quantity = None
        self._order_book_id = None
        self._side = None
        self._position_effect = None
        self._message = None
        self._filled_quantity = None
        self._status = None
        self._frozen_price = None
        self._type = None
        self._avg_price = None
        self._transaction_cost = None

    @classmethod
    def __from_create__(cls, calendar_dt, trading_dt, order_book_id, quantity, side, style, position_effect):
        order = cls()
        order._order_id = next(order.order_id_gen)
        order._calendar_dt = calendar_dt
        order._trading_dt = trading_dt
        order._quantity = quantity
        order._order_book_id = order_book_id
        order._side = side
        order._position_effect = position_effect
        order._message = ""
        order._filled_quantity = 0
        order._status = ORDER_STATUS.PENDING_NEW
        if isinstance(style, LimitOrder):
            order._frozen_price = style.get_limit_price()
            order._type = ORDER_TYPE.LIMIT
        else:
            order._frozen_price = 0.
            order._type = ORDER_TYPE.MARKET
        order._avg_price = 0
        order._transaction_cost = 0
        return order

    @classmethod
    def __from_dict__(cls, order_dict):
        order = cls()
        for persist_key, origin_key in six.iteritems(OrderPersistMap):
            setattr(order, origin_key, order_dict[persist_key])
        return order

    def __to_dict__(self):
        order_dict = {}
        for persist_key, origin_key in six.iteritems(OrderPersistMap):
            order_dict[persist_key] = getattr(self, origin_key)
        return order_dict

    @property
    def order_id(self):
        """
        【int】唯一标识订单的id
        """
        return self._order_id

    @property
    def trading_datetime(self):
        """
        【datetime.datetime】订单的交易日期（对应期货夜盘）
        """
        return self._trading_dt

    @property
    def datetime(self):
        """
        【datetime.datetime】订单创建时间
        """
        return self._calendar_dt

    @property
    def quantity(self):
        """
        【int】订单数量
        """
        return self._quantity

    @property
    def unfilled_quantity(self):
        """
        【int】订单未成交数量
        """
        return self._quantity - self._filled_quantity

    @property
    def order_book_id(self):
        """
        【str】合约代码
        """
        return self._order_book_id

    @property
    def side(self):
        """
        【SIDE】订单方向
        """
        return self._side

    @property
    def position_effect(self):
        """
        【POSITION_EFFECT】订单开平（期货专用）
        """
        return self._position_effect

    @property
    def message(self):
        """
        【str】信息。比如拒单时候此处会提示拒单原因
        """
        return self._message

    @property
    def filled_quantity(self):
        """
        【int】订单已成交数量
        """
        return self._filled_quantity

    @property
    def status(self):
        """
        【ORDER_STATUS】订单状态
        """
        return self._status

    @property
    def price(self):
        """
        【float】订单价格，只有在订单类型为'限价单'的时候才有意义
        """
        return 0 if self.type == ORDER_TYPE.MARKET else self._frozen_price

    @property
    def type(self):
        """
        【ORDER_TYPE】订单类型
        """
        return self._type

    @property
    def avg_price(self):
        """
        【float】成交均价
        """
        return self._avg_price

    @property
    def transaction_cost(self):
        """
        【float】费用
        """
        return self._transaction_cost

    def _is_final(self):
        if self.status == ORDER_STATUS.PENDING_NEW or self.status == ORDER_STATUS.ACTIVE:
            return False
        else:
            return True

    def _is_active(self):
        return self.status == ORDER_STATUS.ACTIVE

    def _active(self):
        self._status = ORDER_STATUS.ACTIVE

    def _fill(self, trade):
        amount = trade.last_quantity
        assert self.filled_quantity + amount <= self.quantity
        new_quantity = self._filled_quantity + amount
        self._avg_price = (self._avg_price * self._filled_quantity + trade.last_price * amount) / new_quantity
        self._transaction_cost += trade.commission + trade.tax
        self._filled_quantity = new_quantity
        if self.unfilled_quantity == 0:
            self._status = ORDER_STATUS.FILLED

    def _mark_rejected(self, reject_reason):
        if not self._is_final():
            self._message = reject_reason
            self._status = ORDER_STATUS.REJECTED
            user_system_log.warn(reject_reason)

    def _mark_cancelled(self, cancelled_reason, user_warn=True):
        if not self._is_final():
            self._message = cancelled_reason
            self._status = ORDER_STATUS.CANCELLED
            if user_warn:
                user_system_log.warn(cancelled_reason)

    def __simple_object__(self):
        return properties(self)


class OrderStyle:
    def get_limit_price(self):
        raise NotImplementedError


class MarketOrder(OrderStyle):
    __repr__ = ORDER_TYPE.MARKET.__repr__

    def get_limit_price(self):
        return None


class LimitOrder(OrderStyle):
    __repr__ = ORDER_TYPE.LIMIT.__repr__

    def __init__(self, limit_price):
        self.limit_price = float(limit_price)

    def get_limit_price(self):
        return self.limit_price



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six

from .base_position import BasePosition
from ...execution_context import ExecutionContext
from ...environment import Environment
from ...const import ACCOUNT_TYPE, SIDE


FuturePersistMap = {
    "_order_book_id": "_order_book_id",
    "_last_price": "_last_price",
    "_market_value": "_market_value",
    "_buy_market_value": "_buy_market_value",
    "_sell_market_value": "_sell_market_value",
    "_buy_trade_value": "_buy_trade_value",
    "_sell_trade_value": "_sell_trade_value",
    "_buy_order_value": "_buy_order_value",
    "_sell_order_value": "_sell_order_value",
    "_buy_order_quantity": "_buy_order_quantity",
    "_sell_order_quantity": "_sell_order_quantity",
    "_buy_trade_quantity": "_buy_trade_quantity",
    "_sell_trade_quantity": "_sell_trade_quantity",
    "_total_orders": "_total_orders",
    "_total_trades": "_total_trades",
    "_is_traded": "_is_traded",
    "_buy_open_order_value": "_buy_open_order_value",
    "_sell_open_order_value": "_sell_open_order_value",
    "_buy_close_order_value": "_buy_close_order_value",
    "_sell_close_order_value": "_sell_close_order_value",
    "_buy_open_order_quantity": "_buy_open_order_quantity",
    "_sell_open_order_quantity": "_sell_open_order_quantity",
    "_buy_close_order_quantity": "_buy_close_order_quantity",
    "_sell_close_order_quantity": "_sell_close_order_quantity",
    "_buy_open_trade_quantity": "_buy_open_trade_quantity",
    "_sell_open_trade_quantity": "_sell_open_trade_quantity",
    "_buy_close_trade_quantity": "_buy_close_trade_quantity",
    "_sell_close_trade_quantity": "_sell_close_trade_quantity",
    "_daily_realized_pnl": "_daily_realized_pnl",
    "_prev_settle_price": "_prev_settle_price",
    "_buy_old_holding_list": "_buy_old_holding_list",
    "_sell_old_holding_list": "_sell_old_holding_list",
    "_buy_today_holding_list": "_buy_today_holding_list",
    "_sell_today_holding_list": "_sell_today_holding_list",
    "_contract_multiplier": "_contract_multiplier",
    "_de_listed_date": "_de_listed_date",
    "_buy_open_transaction_cost": "_buy_open_transaction_cost",
    "_buy_close_transaction_cost": "_buy_close_transaction_cost",
    "_sell_open_transaction_cost": "_sell_open_transaction_cost",
    "_sell_close_transaction_cost": "_sell_close_transaction_cost",
    "_buy_daily_realized_pnl": "_buy_daily_realized_pnl",
    "_sell_daily_realized_pnl": "_sell_daily_realized_pnl",
    "_buy_avg_open_price": "_buy_avg_open_price",
    "_sell_avg_open_price": "_sell_avg_open_price",
}


class FuturePosition(BasePosition):

    # buy_open_order_value:       <float> 买开挂单总值
    # sell_open_order_value:      <float> 卖开挂单总值
    # buy_close_order_value:      <float> 买平挂单总值
    # sell_close_order_value:     <float> 卖平挂单总值
    # buy_open_order_quantity:    <float> 买开挂单量
    # sell_open_order_quantity:   <float> 卖开挂单量
    # buy_close_order_quantity:   <float> 买平挂单量
    # sell_close_order_quantity:  <float> 卖平挂单量
    # buy_open_trade_quantity:    <float> 买开成交量
    # sell_open_trade_quantity:   <float> 卖开成交量
    # buy_close_trade_quantity:   <float> 买平成交量
    # sell_close_trade_quantity:  <float> 买平成交量
    # daily_realized_pnl:         <float> 当日平仓盈亏
    # buy_settle_holding:         <Tuple(price, amount)> 买结算后昨仓
    # sell_settle_holding:        <Tuple(price, amount)> 卖结算后昨仓
    # buy_today_holding_list:     <List<Tuple(price, amount)>> 买当日持仓队列
    # sell_today_holding_list:    <List<Tuple(price, amount)>> 卖当日持仓队列

    def __init__(self, order_book_id):
        super(FuturePosition, self).__init__(order_book_id)
        self._buy_open_order_value = 0.
        self._sell_open_order_value = 0.
        self._buy_close_order_value = 0.
        self._sell_close_order_value = 0.
        self._buy_open_order_quantity = 0
        self._sell_open_order_quantity = 0
        self._buy_close_order_quantity = 0
        self._sell_close_order_quantity = 0
        self._buy_open_trade_quantity = 0
        self._buy_open_trade_value = 0
        self._sell_open_trade_quantity = 0
        self._sell_open_trade_value = 0
        self._buy_close_trade_quantity = 0
        self._buy_close_trade_value = 0
        self._sell_close_trade_quantity = 0
        self._sell_close_trade_value = 0
        self._daily_realized_pnl = 0.
        self._prev_settle_price = 0.
        self._buy_old_holding_list = []         # [(price, amount)]
        self._sell_old_holding_list = []        # [(price, amount)]
        self._buy_today_holding_list = []       # [(price, amount)]
        self._sell_today_holding_list = []      # [(price, amount)]
        instrument = ExecutionContext.get_instrument(self.order_book_id)
        if instrument is None:
            self._contract_multiplier = None
            self._de_listed_date = None
        else:
            self._contract_multiplier = instrument.contract_multiplier
            self._de_listed_date = instrument.de_listed_date

        self._buy_open_transaction_cost = 0.
        self._buy_close_transaction_cost = 0.
        self._sell_open_transaction_cost = 0.
        self._sell_close_transaction_cost = 0.
        self._buy_daily_realized_pnl = 0.
        self._sell_daily_realized_pnl = 0.
        self._buy_avg_open_price = 0.
        self._sell_avg_open_price = 0.
        self._buy_market_value = 0.
        self._sell_market_value = 0.

    @classmethod
    def __from_dict__(cls, position_dict):
        position = cls(position_dict["_order_book_id"])
        for persist_key, origin_key in six.iteritems(FuturePersistMap):
            try:
                setattr(position, origin_key, position_dict[persist_key])
            except KeyError as e:
                if persist_key in ["_buy_avg_open_price", "_sell_avg_open_price"]:
                    # FIXME 对于已有 persist_key 做暂时error handling 处理。
                    setattr(position, origin_key, 0.)
                else:
                    raise e
        return position

    def __to_dict__(self):
        p_dict = {}
        for persist_key, origin_key in six.iteritems(FuturePersistMap):
            p_dict[persist_key] = getattr(self, origin_key)
        return p_dict

    @property
    def buy_open_order_quantity(self):
        """
        【int】买开挂单量
        """
        return self._buy_open_order_quantity

    @property
    def sell_open_order_quantity(self):
        """
        【int】卖开挂单量
        """
        return self._sell_open_order_quantity

    @property
    def buy_close_order_quantity(self):
        """
        【int】买平挂单量
        """
        return self._buy_close_order_quantity

    @property
    def sell_close_order_quantity(self):
        """
        【int】卖平挂单量
        """
        return self._sell_close_order_quantity

    @property
    def daily_realized_pnl(self):
        """
        【float】当日平仓盈亏
        """
        return self._daily_realized_pnl

    @property
    def daily_pnl(self):
        """
        【float】当日盈亏，当日浮动盈亏+当日平仓盈亏
        """
        return self.daily_realized_pnl + self.daily_holding_pnl

    @property
    def daily_holding_pnl(self):
        """
        【float】当日持仓盈亏
        """
        # daily_holding_pnl: < float > 当日持仓盈亏
        return self._market_value + self._sell_holding_cost - self._buy_holding_cost

    @property
    def _buy_daily_holding_pnl(self):
        return (self._last_price - self.buy_avg_holding_price) * self.buy_quantity * self._contract_multiplier

    @property
    def _sell_daily_holding_pnl(self):
        return (self.sell_avg_holding_price - self._last_price) * self.sell_quantity * self._contract_multiplier

    @property
    def buy_daily_pnl(self):
        """
        【float】多头仓位当日盈亏
        """
        return self._buy_daily_holding_pnl + self._buy_daily_realized_pnl

    @property
    def sell_daily_pnl(self):
        """
        【float】空头仓位当日盈亏
        """
        return self._sell_daily_holding_pnl + self._sell_daily_realized_pnl

    @property
    def margin(self):
        """
        【float】仓位总保证金
        """
        # 总保证金
        # TODO 这里之后需要进行修改,需要考虑单向大边的情况
        return self.buy_margin + self.sell_margin

    @property
    def buy_margin(self):
        """
        【float】多头持仓占用保证金
        """
        # buy_margin: < float > 买保证金
        margin_decider = Environment.get_instance().accounts[ACCOUNT_TYPE.FUTURE].margin_decider
        return margin_decider.cal_margin(self.order_book_id, SIDE.BUY, self._buy_holding_cost)

    @property
    def sell_margin(self):
        """
        【float】空头持仓占用保证金
        """
        # sell_margin: < float > 卖保证金
        margin_decider = Environment.get_instance().accounts[ACCOUNT_TYPE.FUTURE].margin_decider
        return margin_decider.cal_margin(self.order_book_id, SIDE.SELL, self._sell_holding_cost)

    @property
    def _buy_old_holding_quantity(self):
        return sum(amount for price, amount in self._buy_old_holding_list)

    @property
    def _sell_old_holding_quantity(self):
        return sum(amount for price, amount in self._sell_old_holding_list)

    @property
    def _buy_today_holding_quantity(self):
        return sum(amount for price, amount in self._buy_today_holding_list)

    @property
    def _sell_today_holding_quantity(self):
        return sum(amount for price, amount in self._sell_today_holding_list)

    @property
    def buy_quantity(self):
        """
        【int】多头持仓
        """
        # 买方向总持仓
        return self._buy_old_holding_quantity + self._buy_today_holding_quantity

    @property
    def sell_quantity(self):
        """
        【int】空头持仓
        """
        # 卖方向总持仓
        return self._sell_old_holding_quantity + self._sell_today_holding_quantity

    @property
    def buy_avg_holding_price(self):
        """
        【float】多头持仓均价
        """
        return 0 if self.buy_quantity == 0 else self._buy_holding_cost / self.buy_quantity / self._contract_multiplier

    @property
    def sell_avg_holding_price(self):
        """
        【float】空头持仓均价
        """
        return 0 if self.sell_quantity == 0 else self._sell_holding_cost / self.sell_quantity / self._contract_multiplier

    @property
    def _buy_closable_quantity(self):
        # 买方向可平仓量
        return self.buy_quantity - self._sell_close_order_quantity

    @property
    def _sell_closable_quantity(self):
        # 卖方向可平仓量
        return self.sell_quantity - self._buy_close_order_quantity

    @property
    def closable_buy_quantity(self):
        """
        【float】可平多头持仓
        """
        return self._buy_closable_quantity

    @property
    def closable_sell_quantity(self):
        """
        【int】可平空头持仓
        """
        return self._sell_closable_quantity

    @property
    def _buy_old_holding_cost(self):
        return self._buy_old_holding_quantity * self._prev_settle_price * self._contract_multiplier

    @property
    def _sell_old_holding_cost(self):
        return self._sell_old_holding_quantity * self._prev_settle_price * self._contract_multiplier

    @property
    def _buy_today_holding_cost(self):
        return sum(p * a * self._contract_multiplier for p, a in self._buy_today_holding_list)

    @property
    def _sell_today_holding_cost(self):
        return sum(p * a * self._contract_multiplier for p, a in self._sell_today_holding_list)

    @property
    def _buy_holding_cost(self):
        return self._buy_old_holding_cost + self._buy_today_holding_cost

    @property
    def _sell_holding_cost(self):
        return self._sell_old_holding_cost + self._sell_today_holding_cost

    @property
    def _quantity(self):
        return self.buy_quantity + self.sell_quantity

    @property
    def _position_value(self):
        # 总保证金 + 当日持仓盈亏 + 当日平仓盈亏
        return self.margin + self.daily_holding_pnl + self.daily_realized_pnl

    @property
    def buy_today_quantity(self):
        """
        【int】多头今仓
        """
        # Buy今仓
        return sum(amount for (price, amount) in self._buy_today_holding_list)

    @property
    def sell_today_quantity(self):
        """
        【int】空头今仓
        """
        # Sell今仓
        return sum(amount for (price, amount) in self._sell_today_holding_list)

    @property
    def _closable_buy_today_quantity(self):
        return self.buy_today_quantity - self._sell_close_order_quantity

    @property
    def _closable_sell_today_quantity(self):
        return self.sell_today_quantity - self._buy_close_order_quantity

    @property
    def buy_pnl(self):
        """
        【float】多头仓位累计盈亏
        """
        return self._sell_close_trade_value - self._buy_open_trade_value + \
               self.buy_quantity * self._last_price * self._contract_multiplier

    @property
    def sell_pnl(self):
        """
        【float】空头仓位累计盈亏
        """
        return self._sell_open_trade_value - self._buy_close_trade_value - \
               self.sell_quantity * self._last_price * self._contract_multiplier

    @property
    def buy_daily_pnl(self):
        """
        【float】多头仓位当日盈亏
        """
        return self._buy_daily_holding_pnl + self._buy_daily_realized_pnl

    @property
    def sell_daily_pnl(self):
        """
        【float】空头仓位当日盈亏
        """
        return self._sell_daily_holding_pnl + self._sell_daily_realized_pnl

    @property
    def _buy_holding_list(self):
        return self._buy_old_holding_list + self._buy_today_holding_list

    @property
    def _sell_holding_list(self):
        return self._sell_old_holding_list + self._sell_today_holding_list

    @property
    def buy_avg_open_price(self):
        """
        【float】多头持仓均价
        """
        return self._buy_avg_open_price

    @property
    def sell_avg_open_price(self):
        """
        【float】空头开仓均价
        """
        return self._sell_avg_open_price

    @property
    def buy_transaction_cost(self):
        """
        【float】多头费用
        """
        return self._buy_open_transaction_cost + self._sell_close_transaction_cost

    @property
    def sell_transaction_cost(self):
        """
        【float】空头费用
        """
        return self._sell_open_transaction_cost + self._buy_close_transaction_cost

    @property
    def transaction_cost(self):
        """
        【float】仓位交易费用
        """
        return self.buy_transaction_cost + self.sell_transaction_cost

    @property
    def buy_market_value(self):
        """
        【float】多头仓位市值加总
        """
        return self._buy_market_value

    @property
    def sell_market_value(self):
        """
        【float】空头仓位市值加总
        """
        return self._sell_market_value

    def _cal_close_today_amount(self, trade_amount, trade_side):
        if trade_side == SIDE.SELL:
            close_today_amount = trade_amount - self._buy_old_holding_quantity
        else:
            close_today_amount = trade_amount - self._sell_old_holding_quantity
        return max(close_today_amount, 0)



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class Tick(object):
    def __init__(self, order_book_id, dt, snapshot):
        self._order_book_id = order_book_id
        self._dt = dt
        self._snapshot = snapshot

    @property
    def order_book_id(self):
        return self._order_book_id

    @property
    def datetime(self):
        return self._dt

    @property
    def open(self):
        return self._snapshot['open']

    @property
    def last(self):
        return self._snapshot['last']

    @property
    def high(self):
        return self._snapshot['high']

    @property
    def low(self):
        return self._snapshot['low']

    @property
    def prev_close(self):
        return self._snapshot['prev_close']

    @property
    def volume(self):
        return self._snapshot['volume']

    @property
    def total_turnover(self):
        return self._snapshot['total_turnover']

    @property
    def open_interest(self):
        return self._snapshot['open_interest']

    @property
    def prev_settlement(self):
        return self._snapshot['prev_settlement']

    @property
    def bid(self):
        return self._snapshot['bid']

    @property
    def bid_volume(self):
        return self._snapshot['bid_volume']

    @property
    def ask(self):
        return self._snapshot['ask']

    @property
    def ask_volume(self):
        return self._snapshot['ask_volume']

    @property
    def limit_up(self):
        return self._snapshot['limit_up']

    @property
    def limit_down(self):
        return self._snapshot['limit_down']



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import click
import errno
import os
import shutil
import ruamel.yaml as yaml
from importlib import import_module

from .utils.click_helper import Date
from .utils.config import parse_config, get_default_config_path, load_config, dump_config


@click.group()
@click.option('-v', '--verbose', count=True)
@click.pass_context
def cli(ctx, verbose):
    ctx.obj["VERBOSE"] = verbose


def entry_point():
    cli(obj={})


@cli.command()
@click.option('-d', '--data-bundle-path', default=os.path.expanduser("~/.rqalpha"), type=click.Path(file_okay=False))
@click.option('--locale', 'locale', type=click.STRING, default="zh_Hans_CN")
def update_bundle(data_bundle_path, locale):
    """
    Sync Data Bundle
    """
    from . import main
    main.update_bundle(data_bundle_path, locale)


@cli.command()
@click.help_option('-h', '--help')
# -- Base Configuration
@click.option('-d', '--data-bundle-path', 'base__data_bundle_path', type=click.Path(exists=True))
@click.option('-f', '--strategy-file', 'base__strategy_file', type=click.Path(exists=True))
@click.option('-s', '--start-date', 'base__start_date', type=Date())
@click.option('-e', '--end-date', 'base__end_date', type=Date())
@click.option('-r', '--rid', 'base__run_id', type=click.STRING)
@click.option('-sc', '--stock-starting-cash', 'base__stock_starting_cash', type=click.FLOAT)
@click.option('-fc', '--future-starting-cash', 'base__future_starting_cash', type=click.FLOAT)
@click.option('-bm', '--benchmark', 'base__benchmark', type=click.STRING, default=None)
@click.option('-sp', '--slippage', 'base__slippage', type=click.FLOAT)
@click.option('-cm', '--commission-multiplier', 'base__commission_multiplier', type=click.FLOAT)
@click.option('-mm', '--margin-multiplier', 'base__margin_multiplier', type=click.FLOAT)
@click.option('-st', '--strategy-type', 'base__strategy_type', type=click.Choice(['stock', 'future', 'stock_future']))
@click.option('-fq', '--frequency', 'base__frequency', type=click.Choice(['1d', '1m']))
@click.option('-me', '--match-engine', 'base__matching_type', type=click.Choice(['current_bar', 'next_bar']))
@click.option('-rt', '--run-type', 'base__run_type', type=click.Choice(['b', 'p']), default="b")
@click.option('--resume', 'base__resume_mode', is_flag=True)
@click.option('--handle-split/--not-handle-split', 'base__handle_split', default=None, help="handle split")
# -- Extra Configuration
@click.option('-l', '--log-level', 'extra__log_level', type=click.Choice(['verbose', 'debug', 'info', 'error', 'none']))
@click.option('--locale', 'extra__locale', type=click.Choice(['cn', 'en']), default="cn")
@click.option('--disable-user-system-log', 'extra__user_system_log_disabled', is_flag=True, help='disable user system log')
@click.option('--extra-vars', 'extra__context_vars', type=click.STRING, help="override context vars")
@click.option("--enable-profiler", "extra__enable_profiler", is_flag=True,
              help="add line profiler to profile your strategy")
@click.option('--config', 'config_path', type=click.STRING, help="config file path")
# -- Mod Configuration
@click.option('-mc', '--mod-config', 'mod_configs', nargs=2, multiple=True, type=click.STRING, help="mod extra config")
@click.option('-p', '--plot/--no-plot', 'mod__analyser__plot', default=None, help="plot result")
@click.option('--plot-save', 'mod__analyser__plot_save_file', default=None, help="save plot to file")
@click.option('--report', 'mod__analyser__report_save_path', type=click.Path(writable=True), help="save report")
@click.option('-o', '--output-file', 'mod__analyser__output_file', type=click.Path(writable=True),
              help="output result pickle file")
@click.option('--progress/--no-progress', 'mod__progress__enabled', default=None, help="show progress bar")
@click.option('--short-stock', 'mod__risk_manager__short_stock', is_flag=True, help="enable stock shorting")
# -- DEPRECATED ARGS && WILL BE REMOVED AFTER VERSION 1.0.0
@click.option('-i', '--init-cash', 'base__stock_starting_cash', type=click.FLOAT)
@click.option('-k', '--kind', 'base__strategy_type', type=click.Choice(['stock', 'future', 'stock_future']))
def run(**kwargs):
    """
    Start to run a strategy
    """
    config_path = kwargs.get('config_path', None)
    if config_path is not None:
        config_path = os.path.abspath(config_path)

    from . import main
    main.run(parse_config(kwargs, config_path))


@cli.command()
@click.option('-d', '--directory', default="./", type=click.Path(), required=True)
def examples(directory):
    """
    Generate example strategies to target folder
    """
    source_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), "examples")

    try:
        shutil.copytree(source_dir, os.path.join(directory, "examples"))
    except OSError as e:
        if e.errno == errno.EEXIST:
            print("Folder examples is exists.")


@cli.command()
@click.argument('result_dict_file', type=click.Path(exists=True), required=True)
@click.option('--show/--hide', 'is_show', default=True)
@click.option('--plot-save', 'plot_save_file', default=None, type=click.Path(), help="save plot result to file")
def plot(result_dict_file, is_show, plot_save_file):
    """
    Draw result DataFrame
    """
    import pandas as pd
    from rqalpha.plot import plot_result

    result_dict = pd.read_pickle(result_dict_file)
    if is_show:
        plot_result(result_dict)
    if plot_save_file:
        plot_result(result_dict, show_windows=False, savefile=plot_save_file)


@cli.command()
@click.argument('result_pickle_file_path', type=click.Path(exists=True), required=True)
@click.argument('target_report_csv_path', type=click.Path(exists=True, writable=True), required=True)
def report(result_pickle_file_path, target_report_csv_path):
    """
    Generate report from backtest output file
    """
    import pandas as pd
    result_dict = pd.read_pickle(result_pickle_file_path)

    from rqalpha.utils.report import generate_report
    generate_report(result_dict, target_report_csv_path)


@cli.command()
@click.option('-v', '--verbose', is_flag=True)
def version(**kwargs):
    """
    Output Version Info
    """
    from rqalpha import version_info
    print("Current Version: ", version_info)


@cli.command()
@click.option('-d', '--directory', default="./", type=click.Path(), required=True)
def generate_config(directory):
    """
    Generate default config file
    """
    default_config = os.path.join(os.path.dirname(os.path.realpath(__file__)), "config_template.yml")
    target_config_path = os.path.abspath(os.path.join(directory, 'config.yml'))
    shutil.copy(default_config, target_config_path)
    print("Config file has been generated in", target_config_path)


# For Mod Cli

system_mod = [
    'simulation',
    'funcat_api',
    'progress',
    'simple_stock_realtime_trade',
    'progressive_output_csv',
    'risk_manager',
    'analyser',
]


@cli.command(context_settings=dict(
    ignore_unknown_options=True,
))
@click.argument('params', nargs=-1)
def install(params):
    """
    Install third-party Mod
    """
    from pip import main as pip_main
    from pip.commands.install import InstallCommand

    params = [param for param in params]

    options, mod_list = InstallCommand().parse_args(params)

    params = ["install"] + params

    for mod_name in mod_list:
        mod_name_index = params.index(mod_name)
        if mod_name in system_mod:
            print('System Mod can not be installed or uninstalled')
            return
        if "rqalpha_mod_" in mod_name:
            lib_name = mod_name
            mod_name = lib_name.replace("rqalpha_mod_", "")
        else:
            lib_name = "rqalpha_mod_" + mod_name
        params[mod_name_index] = lib_name

    # Install Mod
    pip_main(params)

    # Export config
    config_path = get_default_config_path()
    config = load_config(config_path, loader=yaml.RoundTripLoader)

    for mod_name in mod_list:
        if "rqalpha_mod_" in mod_name:
            lib_name = mod_name
            mod_name = lib_name.replace("rqalpha_mod_", "")
        else:
            lib_name = "rqalpha_mod_" + mod_name

        mod = import_module(lib_name)

        mod_config = yaml.load(mod.__mod_config__, yaml.RoundTripLoader)

        config['mod'][mod_name] = mod_config
        config['mod'][mod_name]['lib'] = lib_name
        config['mod'][mod_name]['enabled'] = False
        config['mod'][mod_name]['priority'] = 1000

    dump_config(config_path, config)


@cli.command(context_settings=dict(
    ignore_unknown_options=True,
))
@click.argument('params', nargs=-1)
def uninstall(params):
    """
    Uninstall third-party Mod
    """
    from pip import main as pip_main
    from pip.commands.uninstall import UninstallCommand

    params = [param for param in params]

    options, mod_list = UninstallCommand().parse_args(params)

    params = ["uninstall"] + params

    for mod_name in mod_list:
        mod_name_index = params.index(mod_name)
        if mod_name in system_mod:
            print('System Mod can not be installed or uninstalled')
            return
        if "rqalpha_mod_" in mod_name:
            lib_name = mod_name
            mod_name = lib_name.replace("rqalpha_mod_", "")
        else:
            lib_name = "rqalpha_mod_" + mod_name
        params[mod_name_index] = lib_name

    # Uninstall Mod
    pip_main(params)

    # Remove Mod Config
    config_path = get_default_config_path()
    config = load_config(config_path, loader=yaml.RoundTripLoader)

    for mod_name in mod_list:
        if "rqalpha_mod_" in mod_name:
            mod_name = mod_name.replace("rqalpha_mod_", "")

        del config['mod'][mod_name]

    dump_config(config_path, config)


@cli.command()
def list():
    """
    List all mod configuration
    """
    config_path = get_default_config_path()
    config = load_config(config_path, loader=yaml.RoundTripLoader)

    print(yaml.dump(config['mod'], Dumper=yaml.RoundTripDumper))


@cli.command()
@click.argument('mod_name')
def enable(mod_name):
    """
    enable mod
    """
    if mod_name not in system_mod and "rqalpha_mod_" in mod_name:
        mod_name = mod_name.replace("rqalpha_mod_", "")

    config_path = get_default_config_path()
    config = load_config(config_path, loader=yaml.RoundTripLoader)

    try:
        config['mod'][mod_name]['enabled'] = True
        dump_config(config_path, config)
    except Exception as e:
        pass


@cli.command()
@click.argument('mod_name')
def disable(mod_name):
    """
    disable mod
    """
    if mod_name not in system_mod and "rqalpha_mod_" in mod_name:
        mod_name = mod_name.replace("rqalpha_mod_", "")

    config_path = get_default_config_path()
    config = load_config(config_path, loader=yaml.RoundTripLoader)

    try:
        config['mod'][mod_name]['enabled'] = False
        dump_config(config_path, config)
    except Exception as e:
        pass


if __name__ == '__main__':
    entry_point()



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


def deep_update(from_dict, to_dict):
    for (key, value) in from_dict.items():
        if key in to_dict.keys() and \
                isinstance(to_dict[key], dict) and \
                isinstance(value, dict):
            deep_update(value, to_dict[key])
        else:
            to_dict[key] = value



def init(context):
    context.s1 = "IF88"
    subscribe(context.s1)
    logger.info("Interested in: " + str(context.s1))


def handle_bar(context, bar_dict):
    buy_open(context.s1, 1)


__config__ = {
    "base": {
        "strategy_type": "future",
        "start_date": "2015-01-09",
        "end_date": "2015-03-09",
        "frequency": "1d",
        "matching_type": "next_bar",
        "future_starting_cash": 1000000,
        "commission_multiplier": 0.01,
        "benchmark": None,
    },
    "extra": {
        "log_level": "error",
        # "plot": True,
    },
    "mod": {
        "progress": {
            "enabled": True,
            "priority": 400,
        },
    },
}



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six

from .. import const


class CustomError(object):
    def __init__(self):
        self.stacks = []
        self.msg = None
        self.exc_type = None
        self.exc_val = None
        self.exc_tb = None
        self.error_type = const.EXC_TYPE.NOTSET

    def set_exc(self, exc_type, exc_val, exc_tb):
        self.exc_type = exc_type
        self.exc_val = exc_val
        self.exc_tb = exc_tb

    def set_msg(self, msg):
        self.msg = msg

    def add_stack_info(self, filename, lineno, func_name, code, local_variables={}):
        self.stacks.append((filename, lineno, func_name, code, local_variables))

    @property
    def stacks_length(self):
        return len(self.stacks)

    def __repr__(self):
        if len(self.stacks) == 0:
            return self.msg

        content = ["Traceback (most recent call last):"]
        for filename, lineno, func_name, code, local_variables in self.stacks:
            content.append('  File %s, line %s in %s' % (filename, lineno, func_name))
            content.append('    %s' % (code, ))
            for k, v in six.iteritems(local_variables):
                content.append('    --> %s = %s' % (k, repr(v)))
            content.append('')
        content.append("%s: %s" % (self.exc_type.__name__, self.msg))

        return "\n".join(content)


class CustomException(Exception):
    def __init__(self, error):
        self.error = error


EXC_EXT_NAME = "ricequant_exc"


def patch_user_exc(exc_val, force=False):
    exc_from_type = getattr(exc_val, EXC_EXT_NAME, const.EXC_TYPE.NOTSET)
    if exc_from_type == const.EXC_TYPE.NOTSET or force:
        setattr(exc_val, EXC_EXT_NAME, const.EXC_TYPE.USER_EXC)
    return exc_val


def patch_system_exc(exc_val, force=False):
    exc_from_type = getattr(exc_val, EXC_EXT_NAME, const.EXC_TYPE.NOTSET)
    if exc_from_type == const.EXC_TYPE.NOTSET or force:
        setattr(exc_val, EXC_EXT_NAME, const.EXC_TYPE.SYSTEM_EXC)
    return exc_val


def get_exc_from_type(exc_val):
    exc_from_type = getattr(exc_val, EXC_EXT_NAME, const.EXC_TYPE.NOTSET)
    return exc_from_type


def is_system_exc(exc_val):
    return get_exc_from_type(exc_val) == const.EXC_TYPE.SYSTEM_EXC


def is_user_exc(exc_val):
    return get_exc_from_type(exc_val) == const.EXC_TYPE.USER_EXC


class ModifyExceptionFromType(object):
    def __init__(self, exc_from_type, force=False):
        self.exc_from_type = exc_from_type
        self.force = force

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if exc_val is not None:
            exc_from_type = getattr(exc_val, EXC_EXT_NAME, const.EXC_TYPE.NOTSET)
            if self.force or exc_from_type == const.EXC_TYPE.NOTSET:
                setattr(exc_val, EXC_EXT_NAME, self.exc_from_type)


class RQUserError(Exception):
    ricequant_exc = const.EXC_TYPE.USER_EXC
    pass


class RQInvalidArgument(RQUserError):
    pass


class RQTypeError(RQUserError):
    pass



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from rqalpha.interface import AbstractMod

from .simulation_broker import SimulationBroker
from .simulation_event_source import SimulationEventSource


class SimulationMod(AbstractMod):
    def __init__(self):
        self._env = None

    def start_up(self, env, mod_config):
        self._env = env
        self._env.set_broker(SimulationBroker(self._env))

        event_source = SimulationEventSource(env, env.config.base.account_list)
        env.set_event_source(event_source)

    def tear_down(self, code, exception=None):
        pass



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ...const import DAYS_CNT, ACCOUNT_TYPE
from ...utils import merge_dicts, exclude_benchmark_generator, get_account_type
from ...utils.repr import dict_repr


class MixedAccount(object):
    def __init__(self, accounts):
        self._accounts = exclude_benchmark_generator(accounts)
        self._portfolio = MixedPortfolio([account.portfolio for account in self._accounts.values()])

    def get_portfolio(self, trading_date):
        return MixedPortfolio([account.get_portfolio(trading_date) for account in self._accounts.values()])

    def get_order(self, order_id):
        for account in self._accounts.values():
            order = account.get_order(order_id)
            if order is not None:
                return order

    def get_open_orders(self):
        result = {}
        for account in self._accounts.values():
            result.update(account.get_open_orders())
        return result

    @property
    def portfolio(self):
        return self._portfolio


class MixedPortfolio(object):
    __repr__ = dict_repr

    def __init__(self, portfolio_list):
        self._portfolio_list = portfolio_list

    @property
    def _type(self):
        return ACCOUNT_TYPE.TOTAL

    @property
    def _pid(self):
        return ACCOUNT_TYPE.TOTAL.value

    @property
    def _yesterday_portfolio_value(self):
        return sum(portfolio._yesterday_portfolio_value for portfolio in self._portfolio_list)

    @property
    def daily_returns(self):
        """
        【float】投资组合每日收益率
        """
        return self.daily_pnl / self._yesterday_portfolio_value

    @property
    def starting_cash(self):
        """
        【float】初始资金，为子组合初始资金的加总
        """
        return sum(portfolio.starting_cash for portfolio in self._portfolio_list)

    @property
    def start_date(self):
        """
        【datetime.datetime】策略投资组合的回测/实时模拟交易的开始日期
        """
        for portfolio in self._portfolio_list:
            return portfolio.start_date

    @property
    def frozen_cash(self):
        """
        【float】冻结资金
        """
        return sum(portfolio.frozen_cash for portfolio in self._portfolio_list)

    @property
    def cash(self):
        """
        【float】可用资金，为子组合可用资金的加总
        """
        return sum(portfolio.cash for portfolio in self._portfolio_list)

    @property
    def portfolio_value(self):
        """
        【float】总权益，为子组合总权益加总
        """
        return sum(portfolio.portfolio_value for portfolio in self._portfolio_list)

    @property
    def positions(self):
        """
        【dict】一个包含所有仓位的字典，以order_book_id作为键，position对象作为值，关于position的更多的信息可以在下面的部分找到。
        """
        return MixedPositions(self._portfolio_list)

    @property
    def daily_pnl(self):
        """
        【float】当日盈亏，子组合当日盈亏的加总
        """
        return sum(portfolio.daily_pnl for portfolio in self._portfolio_list)

    @property
    def market_value(self):
        """
        【float】投资组合当前的市场价值，为子组合市场价值的加总
        """
        return sum(portfolio.market_value for portfolio in self._portfolio_list)

    @property
    def pnl(self):
        """
        【float】当前投资组合的累计盈亏
        """
        return self.portfolio_value - self.starting_cash

    @property
    def total_returns(self):
        """
        【float】投资组合至今的累积收益率。计算方法是现在的投资组合价值/投资组合的初始资金
        """
        return self.pnl / self.starting_cash

    @property
    def annualized_returns(self):
        """
        【float】投资组合的年化收益率
        """
        current_date = self._portfolio_list[0]._current_date
        return (1 + self.total_returns) ** (
            DAYS_CNT.DAYS_A_YEAR / float((current_date - self.start_date).days + 1)) - 1

    @property
    def transaction_cost(self):
        """
        【float】总费用
        """
        return sum(portfolio.transaction_cost for portfolio in self._portfolio_list)


class MixedPositions(dict):
    def __init__(self, portfolio_list):
        super(MixedPositions, self).__init__()
        self.portfolio_list = portfolio_list

    def __missing__(self, order_book_id):
        account_type = get_account_type(order_book_id)
        for portfolio in self.portfolio_list:
            if portfolio._type == account_type:
                return portfolio.positions[order_book_id]
        return None

    def __repr__(self):
        keys = []
        for portfolio in self.portfolio_list:
            keys += portfolio.positions.keys()
        return str(sorted(keys))

    def __len__(self):
        return sum(len(portfolio.positions) for portfolio in self.portfolio_list)

    def __iter__(self):
        keys = []
        for portfolio in self.portfolio_list:
            keys += portfolio.positions.keys()
        for key in sorted(keys):
            yield key

    def items(self):
        items = merge_dicts(*[portfolio.positions.items() for portfolio in self.portfolio_list])
        for k in sorted(items.keys()):
            yield k, items[k]

    def keys(self):
        keys = []
        for portfolio in self.portfolio_list:
            keys += list(portfolio.positions.keys())
        return iter(sorted(keys))



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six


class PositionsProxy(object):
    def __init__(self, postions):
        self._postions = postions
        self.__class__.__repr__ = postions.__repr__

    def __repr__(self):
        return str([order_book_id for order_book_id in self.keys()])

    def __getitem__(self, name):
        position = self._postions[name]
        return position._clone()

    def __iter__(self):
        for key in self.keys():
            yield key

    def __len__(self):
        return len(self._postions)

    def items(self):
        for key, value in sorted(six.iteritems(self._postions)):
            yield key, value._clone()

    def keys(self):
        return sorted(self._postions.keys())

    def __setattr__(self, name, value):
        if name not in ["_postions"]:
            raise AttributeError("{} can not modify to {}".format(name, value))
        super(PositionsProxy, self).__setattr__(name, value)


class PortfolioProxy(object):
    def __init__(self, portfolio):
        self._portfolio = portfolio
        self.__class__.__repr__ = portfolio.__repr__

        self.positions = PositionsProxy(portfolio.positions)

    def __getattr__(self, name):
        return getattr(self._portfolio, name)

    def __setattr__(self, name, value):
        if name not in ["_portfolio", "_positions_proxy", "positions"]:
            raise AttributeError("{} can not modify to {}".format(name, value))
        super(PortfolioProxy, self).__setattr__(name, value)



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import time
from threading import Thread

from six.moves.queue import Queue, Empty

from rqalpha.interface import AbstractEventSource
from rqalpha.environment import Environment
from rqalpha.utils.logger import system_log
from rqalpha.events import Event, EVENT
from rqalpha.execution_context import ExecutionContext
from rqalpha.utils import json as json_utils
from .utils import get_realtime_quotes, order_book_id_2_tushare_code, is_holiday_today, is_tradetime_now


class RealtimeEventSource(AbstractEventSource):

    def __init__(self, fps):
        self._env = Environment.get_instance()
        self.fps = fps
        self.event_queue = Queue()

        self.before_trading_fire_date = datetime.date(2000, 1, 1)
        self.after_trading_fire_date = datetime.date(2000, 1, 1)

        self.clock_engine_thread = Thread(target=self.clock_worker)
        self.clock_engine_thread.daemon = True

        self.quotation_engine_thread = Thread(target=self.quotation_worker)
        self.quotation_engine_thread.daemon = True

    def set_state(self, state):
        persist_dict = json_utils.convert_json_to_dict(state.decode('utf-8'))
        self.before_trading_fire_date = persist_dict['before_trading_fire_date']
        self.after_trading_fire_date = persist_dict['after_trading_fire_date']

    def get_state(self):
        return json_utils.convert_dict_to_json({
            "before_trading_fire_date": self.before_trading_fire_date,
            "after_trading_fire_date": self.after_trading_fire_date,
        }).encode('utf-8')

    def quotation_worker(self):
        while True:
            if not is_holiday_today() and is_tradetime_now():
                order_book_id_list = sorted(ExecutionContext.data_proxy.all_instruments("CS").order_book_id.tolist())
                code_list = [order_book_id_2_tushare_code(code) for code in order_book_id_list]

                try:
                    self._env.data_source.realtime_quotes_df = get_realtime_quotes(code_list)
                except Exception as e:
                    system_log.exception("get_realtime_quotes fail")
                    continue

            time.sleep(1)

    def clock_worker(self):
        while True:
            # wait for the first data ready
            if not self._env.data_source.realtime_quotes_df.empty:
                break
            time.sleep(0.1)

        while True:
            time.sleep(self.fps)

            if is_holiday_today():
                time.sleep(60)
                continue

            dt = datetime.datetime.now()

            if dt.strftime("%H:%M:%S") >= "08:30:00" and dt.date() > self.before_trading_fire_date:
                self.event_queue.put((dt, EVENT.BEFORE_TRADING))
                self.before_trading_fire_date = dt.date()
            elif dt.strftime("%H:%M:%S") >= "15:10:00" and dt.date() > self.after_trading_fire_date:
                self.event_queue.put((dt, EVENT.AFTER_TRADING))
                self.after_trading_fire_date = dt.date()

            if is_tradetime_now():
                self.event_queue.put((dt, EVENT.BAR))

    def events(self, start_date, end_date, frequency):
        running = True

        self.clock_engine_thread.start()
        self.quotation_engine_thread.start()

        while running:
            real_dt = datetime.datetime.now()
            while True:
                try:
                    dt, event_type = self.event_queue.get(timeout=1)
                    break
                except Empty:
                    continue

            system_log.debug("real_dt {}, dt {}, event {}", real_dt, dt, event_type)
            yield Event(event_type, real_dt, dt)



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six

from .base_portfolio import BasePortfolio
from ..dividend import Dividend
from ..position import Positions, PositionsClone, FuturePosition
from ...utils.repr import dict_repr


FuturePersistMap = {
    "_yesterday_portfolio_value": "_yesterday_portfolio_value",
    "_yesterday_units": "_yesterday_units",
    "_cash": "_cash",
    "_starting_cash": "_starting_cash",
    "_units": "_units",
    "_start_date": "_start_date",
    "_current_date": "_current_date",
    "_frozen_cash": "_frozen_cash",
    "_total_commission": "_total_commission",
    "_total_tax": "_total_tax",
    "_dividend_receivable": "_dividend_receivable",
    "_dividend_info": "_dividend_info",
    "_daily_transaction_cost": "_daily_transaction_cost",
    "_positions": "_positions",
}


class FuturePortfolioClone(object):
    __repr__ = dict_repr


class FuturePortfolio(BasePortfolio):
    def __init__(self, cash, start_date, account_type):
        super(FuturePortfolio, self).__init__(cash, start_date, account_type)
        self._daily_transaction_cost = 0
        self._positions = Positions(FuturePosition)
        self._portfolio_value = None

    def restore_from_dict_(self, portfolio_dict):
        self._cash = portfolio_dict['_cash']
        self._start_date = portfolio_dict['_start_date']
        self._positions.clear()
        self._dividend_info.clear()
        for persist_key, origin_key in six.iteritems(FuturePersistMap):
            if persist_key == "_dividend_info":
                for order_book_id, dividend_dict in six.iteritems(portfolio_dict[persist_key]):
                    self._dividend_info[order_book_id] = Dividend.__from_dict__(dividend_dict)
            elif persist_key == "_positions":
                for order_book_id, position_dict in six.iteritems(portfolio_dict[persist_key]):
                    self._positions[order_book_id] = FuturePosition.__from_dict__(position_dict)
            else:
                try:
                    setattr(self, origin_key, portfolio_dict[persist_key])
                except KeyError as e:
                    if persist_key in ["_yesterday_units", "_units"]:
                        # FIXME 对于已有 persist_key 做暂时error handling 处理。
                        setattr(self, origin_key, portfolio_dict["_starting_cash"])
                    else:
                        raise e

    def __to_dict__(self):
        p_dict = {}
        for persist_key, origin_key in six.iteritems(FuturePersistMap):
            if persist_key == "_dividend_info":
                p_dict[persist_key] = {oid: dividend.__to_dict__() for oid, dividend in six.iteritems(getattr(self, origin_key))}
            elif persist_key == "_positions":
                p_dict[persist_key] = {oid: position.__to_dict__() for oid, position in six.iteritems(getattr(self, origin_key))}
            else:
                p_dict[persist_key] = getattr(self, origin_key)
        return p_dict

    @property
    def positions(self):
        """
        【dict】一个包含期货子组合仓位的字典，以order_book_id作为键，position对象作为值
        """
        return self._positions

    @property
    def cash(self):
        """
        【float】可用资金
        """
        return self.portfolio_value - self.margin - self.daily_holding_pnl - self.frozen_cash

    @property
    def portfolio_value(self):
        """
        【float】总权益，昨日总权益+当日盈亏
        """
        if self._portfolio_value is None:
            self._portfolio_value = self._yesterday_portfolio_value + self.daily_pnl

        return self._portfolio_value

    # @property
    # def _risk_cash(self):
    #     # 风控资金
    #     risk_cash = self.cash if self.daily_holding_pnl > 0 else self.cash + self.daily_holding_pnl
    #     return risk_cash

    @property
    def buy_margin(self):
        """
        【float】多头保证金
        """
        # 买保证金
        # TODO 这里需要考虑 T TF 这种跨合约单向大边的情况
        # TODO 这里需要考虑 同一个合约跨期单向大边的情况
        return sum(position.buy_margin for position in six.itervalues(self.positions))

    @property
    def sell_margin(self):
        """
        【float】空头保证金
        """
        # 卖保证金
        return sum(position.sell_margin for position in six.itervalues(self.positions))

    @property
    def margin(self):
        """
        【float】已占用保证金
        """
        # 总保证金
        return sum(position.margin for position in six.itervalues(self.positions))

    @property
    def daily_holding_pnl(self):
        """
        【float】当日浮动盈亏
        """
        # 当日持仓盈亏
        return sum(position.daily_holding_pnl for position in six.itervalues(self.positions))

    @property
    def daily_realized_pnl(self):
        """
        【float】当日平仓盈亏
        """
        # 当日平仓盈亏
        return sum(position.daily_realized_pnl for position in six.itervalues(self.positions))

    @property
    def daily_pnl(self):
        """
        【float】当日盈亏，当日浮动盈亏 + 当日平仓盈亏 - 当日费用
        """
        # 当日盈亏
        return self.daily_realized_pnl + self.daily_holding_pnl - self._daily_transaction_cost

    def _clone(self):
        p = FuturePortfolioClone()
        for key in dir(self):
            if "__" in key:
                continue
            if key == "positions":
                ps = PositionsClone(FuturePosition)
                for order_book_id in self.positions:
                    ps[order_book_id] = self.positions[order_book_id]._clone()
                setattr(p, key, ps)
            else:
                setattr(p, key, getattr(self, key))
        return p



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six
import pandas as pd
import numpy as np

from .base_account import BaseAccount
from ..dividend import Dividend
from ...execution_context import ExecutionContext
from ...const import ACCOUNT_TYPE


class BenchmarkAccount(BaseAccount):
    def __init__(self, env, init_cash, start_date):
        super(BenchmarkAccount, self).__init__(env, init_cash, start_date, ACCOUNT_TYPE.BENCHMARK)
        self.benchmark = env.config.base.benchmark

    def before_trading(self):
        portfolio = self.portfolio
        portfolio._yesterday_portfolio_value = portfolio.portfolio_value
        trading_date = ExecutionContext.get_current_trading_dt().date()
        self._handle_dividend_payable(trading_date)

    def bar(self, bar_dict):
        price = bar_dict[self.config.base.benchmark].close
        if np.isnan(price):
            return
        portfolio = self.portfolio
        portfolio._portfolio_value = None
        position = portfolio.positions[self.benchmark]

        if portfolio.market_value == 0:
            trade_quantity = int(portfolio.cash / price)
            delta_value = trade_quantity * price
            commission = 0.0008 * trade_quantity * price
            position._total_commission = commission
            position._buy_trade_quantity = trade_quantity
            position._buy_trade_value = delta_value
            position._market_value = delta_value
            portfolio._cash = portfolio._cash - delta_value - commission
        else:
            position._market_value = position._buy_trade_quantity * price

    def after_trading(self):
        trading_date = ExecutionContext.get_current_trading_dt().date()
        self.portfolio_persist()
        self._handle_dividend_ex_dividend(trading_date)

    def _handle_dividend_payable(self, trading_date):
        """handle dividend payable before trading
        """
        to_delete_dividend = []
        for order_book_id, dividend_info in six.iteritems(self.portfolio._dividend_info):
            dividend_series_dict = dividend_info.dividend_series_dict

            if pd.Timestamp(trading_date) == pd.Timestamp(dividend_series_dict['payable_date']):
                dividend_per_share = dividend_series_dict["dividend_cash_before_tax"] / dividend_series_dict["round_lot"]
                if dividend_per_share > 0 and dividend_info.quantity > 0:
                    dividend_cash = dividend_per_share * dividend_info.quantity
                    self.portfolio._dividend_receivable -= dividend_cash
                    self.portfolio._cash += dividend_cash
                    # user_log.info(_("get dividend {dividend} for {order_book_id}").format(
                    #     dividend=dividend_cash,
                    #     order_book_id=order_book_id,
                    # ))
                    to_delete_dividend.append(order_book_id)

        for order_book_id in to_delete_dividend:
            self.portfolio._dividend_info.pop(order_book_id, None)

    def _handle_dividend_ex_dividend(self, trading_date):
        data_proxy = ExecutionContext.get_data_proxy()
        for order_book_id, position in six.iteritems(self.portfolio.positions):
            dividend_series = data_proxy.get_dividend_by_book_date(order_book_id, trading_date)
            if dividend_series is None:
                continue

            dividend_series_dict = {
                'book_closure_date': dividend_series['book_closure_date'],
                'ex_dividend_date': dividend_series['ex_dividend_date'],
                'payable_date': dividend_series['payable_date'],
                'dividend_cash_before_tax': dividend_series['dividend_cash_before_tax'],
                'round_lot': dividend_series['round_lot']
            }

            dividend_per_share = dividend_series_dict["dividend_cash_before_tax"] / dividend_series_dict["round_lot"]
            self.portfolio._dividend_info[order_book_id] = Dividend(order_book_id, position._quantity, dividend_series_dict)
            self.portfolio._dividend_receivable += dividend_per_share * position._quantity



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six
import os
import ruamel.yaml as yaml
import datetime
import logbook
import locale
from pprint import pformat
import codecs
import shutil

from . import RqAttrDict, logger
from .exception import patch_user_exc
from .logger import user_log, user_system_log, system_log, std_log, user_std_handler
from ..const import ACCOUNT_TYPE, MATCHING_TYPE, RUN_TYPE, PERSIST_MODE
from ..utils.i18n import gettext as _, localization
from ..utils.dict_func import deep_update
from ..mod.utils import mod_config_value_parse


def load_config(config_path, loader=yaml.Loader, verify_version=True):
    if not os.path.exists(config_path):
        system_log.error(_("config.yml not found in {config_path}").format(config_path))
        return False
    with codecs.open(config_path, encoding="utf-8") as stream:
        config = yaml.load(stream, loader)
    if verify_version:
        config = config_version_verify(config, config_path)
    return config


def dump_config(config_path, config, dumper=yaml.RoundTripDumper):
    with codecs.open(config_path, mode='w', encoding='utf-8') as file:
        file.write(yaml.dump(config, Dumper=dumper))


def get_default_config_path():
    config_template_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "../config_template.yml")
    default_config_path = os.path.abspath(os.path.expanduser("~/.rqalpha/config.yml"))
    if not os.path.exists(default_config_path):
        dir_path = os.path.dirname(default_config_path)
        if not os.path.exists(dir_path):
            os.makedirs(dir_path)
        shutil.copy(config_template_path, default_config_path)
    return default_config_path


def config_version_verify(config, config_path):
    config_template_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "../config_template.yml")
    default_config = load_config(config_template_path, verify_version=False)
    config_version = config.get("version", None)
    if config_version != default_config["version"]:
        back_config_file_path = config_path + "." + datetime.datetime.now().date().strftime("%Y%m%d") + ".bak"
        shutil.move(config_path, back_config_file_path)
        shutil.copy(config_template_path, config_path)

        system_log.warning(_("""
Your current config file {config_file_path} is too old and may cause RQAlpha running error.
RQAlpha has replaced the config file with the newest one.
the backup config file has been saved in {back_config_file_path}.
        """).format(config_file_path=config_path, back_config_file_path=back_config_file_path))
        config = default_config
    return config


def set_locale(lc):
    # FIXME: It should depends on the system and locale config
    try:
        locale.setlocale(locale.LC_ALL, "en_US.UTF-8")
        locale.setlocale(locale.LC_CTYPE, "en_US.UTF-8")
        os.environ['TZ'] = 'Asia/Shanghai'
    except Exception as e:
        if os.name != 'nt':
            raise
    localization.set_locale([lc])


def parse_config(config_args, config_path=None, click_type=True, source_code=None):
    mod_configs = config_args.pop("mod_configs", [])
    for cfg, value in mod_configs:
        key = "mod__{}".format(cfg.replace(".", "__"))
        config_args[key] = mod_config_value_parse(value)

    set_locale(config_args.get("extra__locale", None))

    config_path = get_default_config_path() if config_path is None else os.path.abspath(config_path)

    config = load_config(config_path)

    if click_type:
        for key, value in six.iteritems(config_args):
            if key in ["config_path"]:
                continue
            if config_args[key] is not None:
                keys = key.split("__")
                keys.reverse()
                sub_config = config[keys.pop()]
                while True:
                    if len(keys) == 0:
                        break
                    k = keys.pop()
                    if len(keys) == 0:
                        sub_config[k] = value
                    else:
                        sub_config = sub_config[k]
    else:
        deep_update(config_args, config)

    config = parse_user_config(config, source_code)

    config = RqAttrDict(config)

    base_config = config.base
    extra_config = config.extra

    set_locale(extra_config.locale)

    if isinstance(base_config.start_date, six.string_types):
        base_config.start_date = datetime.datetime.strptime(base_config.start_date, "%Y-%m-%d")
    if isinstance(base_config.start_date, datetime.datetime):
        base_config.start_date = base_config.start_date.date()
    if isinstance(base_config.end_date, six.string_types):
        base_config.end_date = datetime.datetime.strptime(base_config.end_date, "%Y-%m-%d")
    if isinstance(base_config.end_date, datetime.datetime):
        base_config.end_date = base_config.end_date.date()
    if base_config.commission_multiplier < 0:
        raise patch_user_exc(ValueError(_("invalid commission multiplier value: value range is [0, +∞)")))
    if base_config.margin_multiplier <= 0:
        raise patch_user_exc(ValueError(_("invalid margin multiplier value: value range is (0, +∞]")))

    if base_config.data_bundle_path is None:
        base_config.data_bundle_path = os.path.expanduser("~/.rqalpha")

    base_config.data_bundle_path = os.path.abspath(base_config.data_bundle_path)

    if os.path.basename(base_config.data_bundle_path) != "bundle":
        base_config.data_bundle_path = os.path.join(base_config.data_bundle_path, "./bundle")

    if not os.path.exists(base_config.data_bundle_path):
        system_log.error(
            _("data bundle not found in {bundle_path}. Run `rqalpha update_bundle` to download data bundle.").format(
                bundle_path=base_config.data_bundle_path))
        return

    if source_code is None and not os.path.exists(base_config.strategy_file):
        system_log.error(
            _("strategy file not found in {strategy_file}").format(strategy_file=base_config.strategy_file))
        return

    base_config.run_type = parse_run_type(base_config.run_type)
    base_config.account_list = gen_account_list(base_config.strategy_type)
    base_config.matching_type = parse_matching_type(base_config.matching_type)
    base_config.persist_mode = parse_persist_mode(base_config.persist_mode)

    if extra_config.log_level.upper() != "NONE":
        user_log.handlers.append(user_std_handler)
        if not extra_config.user_system_log_disabled:
            user_system_log.handlers.append(user_std_handler)

    if extra_config.context_vars:
        import base64
        import json
        extra_config.context_vars = json.loads(base64.b64decode(extra_config.context_vars).decode('utf-8'))

    if base_config.stock_starting_cash < 0:
        raise patch_user_exc(ValueError(_('invalid stock starting cash: {}').format(base_config.stock_starting_cash)))

    if base_config.future_starting_cash < 0:
        raise patch_user_exc(ValueError(_('invalid future starting cash: {}').format(base_config.future_starting_cash)))

    if base_config.stock_starting_cash + base_config.future_starting_cash == 0:
        raise patch_user_exc(ValueError(_('stock starting cash and future starting cash can not be both 0.')))

    system_log.level = getattr(logbook, extra_config.log_level.upper(), logbook.NOTSET)
    std_log.level = getattr(logbook, extra_config.log_level.upper(), logbook.NOTSET)
    user_log.level = getattr(logbook, extra_config.log_level.upper(), logbook.NOTSET)
    user_system_log.level = getattr(logbook, extra_config.log_level.upper(), logbook.NOTSET)

    if base_config.frequency == "1d":
        logger.DATETIME_FORMAT = "%Y-%m-%d"

    system_log.debug("\n" + pformat(config))

    return config


def parse_user_config(config, source_code=None):
    try:
        if source_code is None:
            with codecs.open(config["base"]["strategy_file"], encoding="utf-8") as f:
                source_code = f.read()

        scope = {}

        code = compile(source_code, config["base"]["strategy_file"], 'exec')
        six.exec_(code, scope)

        __config__ = scope.get("__config__", {})

        deep_update(__config__, config)

        for sub_key, sub_dict in six.iteritems(__config__):
            if sub_key not in config["whitelist"]:
                continue
            deep_update(sub_dict, config[sub_key])

    except Exception as e:
        system_log.error(_('in parse_user_config, exception: {e}').format(e=e))
    finally:
        return config


def gen_account_list(account_list_str):
    assert isinstance(account_list_str, six.string_types)
    account_list = account_list_str.split("_")
    return [parse_account_type(account_str) for account_str in account_list]


def parse_account_type(account_type_str):
    assert isinstance(account_type_str, six.string_types)
    if account_type_str == "stock":
        return ACCOUNT_TYPE.STOCK
    elif account_type_str == "future":
        return ACCOUNT_TYPE.FUTURE


def parse_matching_type(me_str):
    assert isinstance(me_str, six.string_types)
    if me_str == "current_bar":
        return MATCHING_TYPE.CURRENT_BAR_CLOSE
    elif me_str == "next_bar":
        return MATCHING_TYPE.NEXT_BAR_OPEN
    else:
        raise NotImplementedError


def parse_run_type(rt_str):
    assert isinstance(rt_str, six.string_types)
    if rt_str == "b":
        return RUN_TYPE.BACKTEST
    elif rt_str == "p":
        return RUN_TYPE.PAPER_TRADING
    elif rt_str == "CTP":
        return RUN_TYPE.CTP
    else:
        raise NotImplementedError


def parse_persist_mode(persist_mode):
    assert isinstance(persist_mode, six.string_types)
    if persist_mode == 'real_time':
        return PERSIST_MODE.REAL_TIME
    elif persist_mode == 'on_crash':
        return PERSIST_MODE.ON_CRASH
    else:
        raise RuntimeError(_('unknown persist mode: {persist_mode}').format(persist_mode=persist_mode))



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from rqalpha.const import SIDE, POSITION_EFFECT
from rqalpha.utils.i18n import gettext as _
from rqalpha.execution_context import ExecutionContext


class FrontendValidator(object):
    def __init__(self, config):
        self.config = config

    def order_pipeline(self, account, order):
        order_book_id = order.order_book_id
        bar_dict = ExecutionContext.get_current_bar_dict()
        portfolio = account.portfolio
        position = portfolio.positions[order_book_id]
        bar = bar_dict[order_book_id]

        if not self.validate_trading(order, bar):
            return False
        if not self.validate_available_cash(order, account, bar):
            return False
        if self.config.short_stock:
            # 如果开启做空，则不验证仓位是否足够
            return True
        if not self.validate_available_position(order, position):
            return False
        return True

    def validate_trading(self, order, bar):
        order_book_id = order.order_book_id
        trading_date = ExecutionContext.get_current_trading_dt().date()

        if bar.isnan:
            """
            只有未上市/已退市，对应的bar才为NaN
            """
            instrument = ExecutionContext.get_instrument(order_book_id)
            if trading_date < instrument.listed_date.date():
                order._mark_rejected(_("Order Rejected: {order_book_id} is not listed!").format(
                    order_book_id=order_book_id,
                ))
            elif trading_date > instrument.de_listed_date.date():
                order._mark_rejected(_("Order Rejected: {order_book_id} has been delisted!").format(
                    order_book_id=order_book_id,
                ))
            else:
                order._mark_rejected(_("Order Rejected: {order_book_id} is not trading!").format(
                    order_book_id=order_book_id,
                ))
            return False
        elif not bar.is_trading:
            """
            如果bar.is_trading为False，还需要判断是否为停盘，如果不是停牌，则说明交易量为0.
            """
            if bar.suspended:
                order._mark_rejected(_("Order Rejected: {order_book_id} is suspended!").format(
                    order_book_id=order_book_id,
                ))
                return False
        return True

    def validate_available_cash(self, order, account, bar):
        raise NotImplementedError

    def validate_available_position(self, order, position):
        raise NotImplementedError


class StockFrontendValidator(FrontendValidator):
    def validate_available_cash(self, order, account, bar):
        if not self.config.available_cash:
            return True
        if order.side != SIDE.BUY:
            return True
        # 检查可用资金是否充足
        cost_money = order._frozen_price * order.quantity
        if cost_money > account.portfolio.cash:
            order._mark_rejected(_(
                "Order Rejected: not enough money to buy {order_book_id}, needs {cost_money:.2f}, cash {cash:.2f}").format(
                order_book_id=order.order_book_id,
                cost_money=cost_money,
                cash=account.portfolio.cash,
            ))
            return False
        return True

    def validate_available_position(self, order, position):
        if not self.config.available_position:
            return True
        if order.side != SIDE.SELL:
            return True
        if order.quantity > position.sellable:
            order._mark_rejected(_(
                "Order Rejected: not enough stock {order_book_id} to sell, you want to sell {quantity}, sellable {sellable}").format(
                order_book_id=order.order_book_id,
                quantity=order.quantity,
                sellable=position.sellable,
            ))
            return False
        return True


class FutureFrontendValidator(FrontendValidator):
    def validate_available_cash(self, order, account, bar):
        if not self.config.available_cash:
            return True
        if order.position_effect != POSITION_EFFECT.OPEN:
            return True
        contract_multiplier = bar.instrument.contract_multiplier
        cost_money = account.margin_decider.cal_margin(order.order_book_id, order.side, order._frozen_price * order.quantity * contract_multiplier)
        if cost_money > account.portfolio.cash:
            order._mark_rejected(_(
                "Order Rejected: not enough money to buy {order_book_id}, needs {cost_money:.2f}, cash {cash:.2f}").format(
                order_book_id=order.order_book_id,
                cost_money=cost_money,
                cash=account.portfolio.cash,
            ))
            return False
        return True

    def validate_available_position(self, order, position):
        if not self.config.available_position:
            return True
        if order.position_effect != POSITION_EFFECT.CLOSE:
            return True
        if order.side == SIDE.BUY and order.quantity > position._sell_closable_quantity:
            order._mark_rejected(_(
                "Order Rejected: not enough securities {order_book_id} to buy close, target sell quantity is {quantity}, sell_closable_quantity {closable}").format(
                order_book_id=order.order_book_id,
                quantity=order.quantity,
                closable=position._sell_closable_quantity,
            ))
            return False
        elif order.side == SIDE.SELL and order.quantity > position._buy_closable_quantity:
            order._mark_rejected(_(
                "Order Rejected: not enough securities {order_book_id} to sell close, target sell quantity is {quantity}, buy_closable_quantity {closable}").format(
                order_book_id=order.order_book_id,
                quantity=order.quantity,
                closable=position._buy_closable_quantity,
            ))
            return False
        return True



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc


from six import with_metaclass


class AbstractStrategyLoader(with_metaclass(abc.ABCMeta)):
    """
    策略加载器，其主要作用是加载策略，并将策略运行所需要的域环境传递给策略执行代码。

    在扩展模块中，可以通过调用 ``env.set_strategy_loader`` 来替换默认的策略加载器。
    """
    @abc.abstractmethod
    def load(self, strategy, scope):
        """
        【Required】

        load 函数负责组装策略代码和策略代码所在的域，并输出最终组装好的可执行域。

        :param str strategy: 策略标识符，对应 ``config.base.strategy_file``，相应的命令行参数为 ``-f``。
        :param dict scope: 策略代码运行环境，在传入时，包含了所有基础API。
            通过在 scope 中添加函数可以实现自定义API；通过覆盖 scope 中相应的函数，可以覆盖原API。

        :return: scope，其中应包含策略相应函数，如 ``init``, ``before_trading`` 等
        """
        raise NotImplementedError


class AbstractEventSource(with_metaclass(abc.ABCMeta)):
    """
    事件源接口。RQAlpha 从此对象中获取事件，驱动整个事件循环。

    在扩展模块中，可以通过调用 ``env.set_event_source`` 来替换默认的事件源。
    """
    @abc.abstractmethod
    def events(self, start_date, end_date, frequency):
        """
        【Required】

        扩展 EventSource 必须实现 events 函数。

        events 是一个 event generator, 在相应事件的时候需要以如下格式来传递事件

        .. code-block:: python

            yield trading_datetime, calendar_datetime, EventEnum

        其中 trading_datetime 为基于交易日的 datetime, calendar_datetime 为基于自然日的 datetime (因为夜盘的存在，交易日和自然日未必相同)

        EventEnum 为 :class:`~Events`

        :param datetime.date start_date: 起始日期, 系统会将 `config.base.start_date` 传递 events 函数
        :param datetime.date end_date: 结束日期，系统会将 `config.base.end_date` 传递给 events 函数
        :param str frequency: 周期频率，`1d` 表示日周期, `1m` 表示分钟周期

        :return: None
        """
        raise NotImplementedError


class AbstractDataSource(object):
    """
    数据源接口。RQAlpha 中通过 :class:`DataProxy` 进一步进行了封装，向上层提供更易用的接口。

    在扩展模块中，可以通过调用 ``env.set_data_source`` 来替换默认的数据源。可参考 :class:`BaseDataSource`。
    """
    def get_all_instruments(self):
        """
        获取所有Instrument。

        :return: list[:class:`~Instrument`]
        """
        raise NotImplementedError

    def get_trading_calendar(self):
        """
        获取交易日历

        :return: list[`pandas.Timestamp`]
        """
        raise NotImplementedError

    def get_yield_curve(self, start_date, end_date, tenor=None):
        """
        获取国债利率

        :param pandas.Timestamp str start_date: 开始日期
        :param pandas.Timestamp end_date: 结束日期
        :param str tenor: 利率期限

        :return: pandas.DataFrame, [start_date, end_date]
        """
        raise NotImplementedError

    def get_dividend(self, order_book_id, adjusted=True):
        """
        获取股票/基金分红信息

        :param str order_book_id: 合约名
        :param bool adjusted: 是否经过前复权处理
        :return:
        """
        raise NotImplementedError

    def get_split(self, order_book_id):
        """
        获取拆股信息

        :param str order_book_id: 合约名

        :return: `pandas.DataFrame`
        """

        raise NotImplementedError

    def get_bar(self, instrument, dt, frequency):
        """
        根据 dt 来获取对应的 Bar 数据

        :param instrument: 合约对象
        :type instrument: :class:`~Instrument`

        :param datetime.datetime dt: calendar_datetime

        :param str frequency: 周期频率，`1d` 表示日周期, `1m` 表示分钟周期

        :return: `numpy.ndarray` | `dict`
        """
        raise NotImplementedError

    def get_settle_price(self, instrument, date):
        """
        获取期货品种在 date 的结算价

        :param instrument: 合约对象
        :type instrument: :class:`~Instrument`

        :param datetime.date date: 结算日期

        :return: `str`
        """
        raise NotImplementedError

    def history_bars(self, instrument, bar_count, frequency, fields, dt, skip_suspended=True):
        """
        获取历史数据

        :param instrument: 合约对象
        :type instrument: :class:`~Instrument`

        :param int bar_count: 获取的历史数据数量
        :param str frequency: 周期频率，`1d` 表示日周期, `1m` 表示分钟周期
        :param str fields: 返回数据字段

        =========================   ===================================================
        fields                      字段名
        =========================   ===================================================
        datetime                    时间戳
        open                        开盘价
        high                        最高价
        low                         最低价
        close                       收盘价
        volume                      成交量
        total_turnover              成交额
        datetime                    int类型时间戳
        open_interest               持仓量（期货专用）
        basis_spread                期现差（股指期货专用）
        settlement                  结算价（期货日线专用）
        prev_settlement             结算价（期货日线专用）
        =========================   ===================================================

        :param datetime.datetime dt: 时间

        :param bool skip_suspended: 是否跳过停牌日

        :return: `numpy.ndarray`

        """
        raise NotImplementedError

    def current_snapshot(self, instrument, frequency, dt):
        """
        获得当前市场快照数据。只能在日内交易阶段调用，获取当日调用时点的市场快照数据。
        市场快照数据记录了每日从开盘到当前的数据信息，可以理解为一个动态的day bar数据。
        在目前分钟回测中，快照数据为当日所有分钟线累积而成，一般情况下，最后一个分钟线获取到的快照数据应当与当日的日线行情保持一致。
        需要注意，在实盘模拟中，该函数返回的是调用当时的市场快照情况，所以在同一个handle_bar中不同时点调用可能返回的数据不同。
        如果当日截止到调用时候对应股票没有任何成交，那么snapshot中的close, high, low, last几个价格水平都将以0表示。

        :param instrument: 合约对象
        :type instrument: :class:`~Instrument`

        :param str frequency: 周期频率，`1d` 表示日周期, `1m` 表示分钟周期
        :param datetime.datetime dt: 时间

        :return: :class:`~Snapshot`
        """
        raise NotImplementedError

    def get_trading_minutes_for(self, instrument, trading_dt):
        """

        获取证券某天的交易时段，用于期货回测

        :param instrument: 合约对象
        :type instrument: :class:`~Instrument`

        :param datetime.datetime trading_dt: 交易日。注意期货夜盘所属交易日规则。

        :return: list[`datetime.datetime`]
        """
        raise NotImplementedError

    def available_data_range(self, frequency):
        """
        此数据源能提供数据的时间范围

        :param str frequency: 周期频率，`1d` 表示日周期, `1m` 表示分钟周期

        :return: (earliest, latest)
        """
        raise NotImplementedError

    def get_future_info(self, order_book_id, hedge_type):
        """
        获取期货合约手续费、保证金等数据

        :param str order_book_id: 合约名
        :param HEDGE_TYPE hedge_type: 枚举类型，账户对冲类型
        :return: dict
        """
        raise NotImplementedError


class AbstractBroker(with_metaclass(abc.ABCMeta)):
    """
    券商接口。

    RQAlpha 将产生的订单提交给此对象，此对象负责对订单进行撮合（不论是自行撮合还是委托给外部的真实交易所），
    并通过 ``EVENT.ORDER_*`` 及 ``EVENT.TRADE`` 事件将撮合结果反馈进入 RQAlpha。

    在扩展模块中，可以通过调用 ``env.set_broker`` 来替换默认的 Broker。
    """
    @abc.abstractmethod
    def get_accounts(self):
        """
        [Required]

        获取账号信息。系统初始化时，RQAlpha 会调用此接口，获取账户信息。

        RQAlpha 支持混合策略，因此返回的账户信息为一个字典(dict)，key 为账户类型（``rqalpha.const.ACCOUNT_TYPE``)，
        value 为对应的 :class:`~Account` 对象。

        :return: dict
        """
        raise NotImplementedError

    @abc.abstractmethod
    def submit_order(self, order):
        """
        【Required】

        提交订单。在当前版本，RQAlpha 会生成 :class:`~Order` 对象，再通过此接口提交到 Broker。
        TBD: 由 Broker 对象生成 Order 并返回？
        """
        raise NotImplementedError

    @abc.abstractmethod
    def cancel_order(self, order):
        """
        【Required】

        撤单。

        :param order: 订单
        :type order: :class:`~Order`
        """
        raise NotImplementedError

    @abc.abstractmethod
    def get_open_orders(self):
        """
        【Required】

        获得当前未完成的订单。

        :return: list[:class:`~Order`]
        """
        raise NotImplementedError

    def before_trading(self):
        """
        【Optional】

        RQAlpha 会在 `EVENT.BEFORE_TRADING` 事件被触发前调用此接口。
        """
        pass

    def after_trading(self):
        """
        【Optional】

        RQAlpha 会在 `EVENT.AFTER_TRADING` 事件被触发前调用此接口。
        """
        pass

    def bar(self, bar_dict):
        """
        【Optional】

        RQAlpha 会在 `EVENT.BAR` 事件被出发前调用此接口。

        Broker 根据自己的业务场景来选择是否实现，比如说自带撮合引擎的Broker，会通过 `update` 函数来触发撮合。

        :param calendar_dt: 实际时间
        :param trading_dt: 交易时间
        :param bar_dict: dict[:class:`~BarObject`]
        """
        pass


class AbstractMod(with_metaclass(abc.ABCMeta)):
    """
    扩展模块接口。
    """
    @abc.abstractmethod
    def start_up(self, env, mod_config):
        """
        RQAlpha 在系统启动时会调用此接口；在此接口中，可以通过调用 ``env`` 的相应方法来覆盖系统默认组件。

        :param env: 系统环境
        :type env: :class:`~Environment`
        :param mod_config: 模块配置参数
        """
        raise NotImplementedError

    def tear_down(self, code, exception=None):
        """
        RQAlpha 在系统退出前会调用此接口。

        :param code: 退出代码
        :type code: rqalpha.const.EXIT_CODE
        :param exception: 如果在策略执行过程中出现错误，此对象为相应的异常对象
        """
        raise NotImplementedError


class AbstractPersistProvider(with_metaclass(abc.ABCMeta)):
    """
    持久化服务提供者接口。

    扩展模块可以通过调用 ``env.set_persist_provider`` 接口来替换默认的持久化方案。
    """
    @abc.abstractmethod
    def store(self, key, value):
        """
        store

        :param str key:
        :param bytes value:
        :return:
        """
        raise NotImplementedError

    @abc.abstractmethod
    def load(self, key):
        """
        :param str key:
        :return: bytes 如果没有对应的值，返回 None
        """
        raise NotImplementedError


class Persistable(with_metaclass(abc.ABCMeta)):
    @abc.abstractmethod
    def get_state(self):
        """
        :return: bytes
        """
        raise NotImplementedError

    @abc.abstractmethod
    def set_state(self, state):
        """
        :param state: bytes
        :return:
        """
        raise NotImplementedError

    @classmethod
    def __subclasshook__(cls, C):
        if cls is Persistable:
            if (any("get_state" in B.__dict__ for B in C.__mro__) and
                    any("set_state" in B.__dict__ for B in C.__mro__)):
                return True
        return NotImplemented



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import click

from rqalpha.interface import AbstractMod
from rqalpha.events import EVENT


class ProgressMod(AbstractMod):
    def __init__(self):
        self._env = None
        self.progress_bar = None

    def start_up(self, env, mod_config):
        self._env = env
        env.event_bus.add_listener(EVENT.POST_AFTER_TRADING, self._tick)
        env.event_bus.add_listener(EVENT.POST_SYSTEM_INIT, self._init)

    def _init(self):
        trading_length = len(self._env.config.base.trading_calendar)
        self.progress_bar = click.progressbar(length=trading_length, show_eta=False)

    def _tick(self):
        self.progress_bar.update(1)

    def tear_down(self, success, exception=None):
        self.progress_bar.render_finish()


def load_mod():
    return ProgressMod()



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc

from six import with_metaclass

from ..const import SIDE
from ..utils.exception import patch_user_exc
from ..utils.i18n import gettext as _


def init_slippage(rate=0):
    # 未来可能会有多种slippage模型，目前只返回 FixedSlippage
    return PriceRatioSlippage(rate)


class BaseSlippage(with_metaclass(abc.ABCMeta)):
    @abc.abstractmethod
    def get_trade_price(self, order, price):
        raise NotImplementedError


class PriceRatioSlippage(BaseSlippage):
    def __init__(self, rate=0.):
        # Rate必须在0~1之间
        if 0 <= rate < 1:
            self.rate = rate
        else:
            raise patch_user_exc(ValueError(_("invalid slippage rate value: value range is [0, 1)")))

    def get_trade_price(self, order, price):
        return price + price * self.rate * (1 if order.side == SIDE.BUY else -1)


# class FixedSlippage(BaseSlippage):
#     def __init__(self, rate=0.):
#         self.rate = rate
#
#     def get_trade_price(self, price):
#         return price + price * self.rate * (0.5 if order.side == SIDE.BUY else -0.5)



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six

from .base_position import BasePosition
from ...execution_context import ExecutionContext
from ...const import ACCOUNT_TYPE


StockPersistMap = {
    "_order_book_id": "_order_book_id",
    "_last_price": "_last_price",
    "_market_value": "_market_value",
    "_buy_trade_value": "_buy_trade_value",
    "_sell_trade_value": "_sell_trade_value",
    "_buy_order_value": "_buy_order_value",
    "_sell_order_value": "_sell_order_value",
    "_buy_order_quantity": "_buy_order_quantity",
    "_sell_order_quantity": "_sell_order_quantity",
    "_buy_trade_quantity": "_buy_trade_quantity",
    "_sell_trade_quantity": "_sell_trade_quantity",
    "_total_orders": "_total_orders",
    "_total_trades": "_total_trades",
    "_is_traded": "_is_traded",
    "_buy_today_holding_quantity": "_buy_today_holding_quantity",
    "_avg_price": "_avg_price",
    "_de_listed_date": "_de_listed_date",
    "_transaction_cost": "_transaction_cost",
}


class StockPosition(BasePosition):

    def __init__(self, order_book_id):
        super(StockPosition, self).__init__(order_book_id)
        self._buy_today_holding_quantity = 0        # int   T+1,所以记录下来该股票今天的买单量
        self._avg_price = 0.                        # float	获得该持仓的买入均价，计算方法为每次买入的数量做加权平均。
        instrument = ExecutionContext.get_instrument(self.order_book_id)
        self._de_listed_date = None if instrument is None else instrument.de_listed_date
        self._transaction_cost = 0.

    @classmethod
    def __from_dict__(cls, position_dict):
        position = cls(position_dict["_order_book_id"])
        for persist_key, origin_key in six.iteritems(StockPersistMap):
            setattr(position, origin_key, position_dict[persist_key])
        return position

    def __to_dict__(self):
        p_dict = {}
        for persist_key, origin_key in six.iteritems(StockPersistMap):
            p_dict[persist_key] = getattr(self, origin_key)
        return p_dict

    @property
    def _position_value(self):
        return self._market_value

    @property
    def quantity(self):
        """
        【int】当前持仓股数
        """
        return self._quantity

    @property
    def bought_quantity(self):
        """
        【int】该证券的总买入股数，例如：如果你的投资组合并没有任何平安银行的成交，那么平安银行这个股票的仓位就是0
        """
        return self._buy_trade_quantity

    @property
    def sold_quantity(self):
        """
        【int】该证券的总卖出股数，例如：如果你的投资组合曾经买入过平安银行股票200股并且卖出过100股，那么这个属性会返回100
        """
        return self._sell_trade_quantity

    @property
    def bought_value(self):
        """
        【float】该证券的总买入的价值，等于每一个该证券的 买入成交价 * 买入股数 总和
        """
        return self._buy_trade_value

    @property
    def sold_value(self):
        """
        【float】该证券的总卖出价值，等于每一个该证券的 卖出成交价 * 卖出股数 总和
        """
        return self._sell_trade_value

    @property
    def average_cost(self):
        """
        【已弃用】请使用 avg_price 获取持仓买入均价
        """
        return self._avg_price

    @property
    def avg_price(self):
        """
        【float】获得该持仓的买入均价，计算方法为每次买入的数量做加权平均
        """
        return self._avg_price

    @property
    def sellable(self):
        """
        【int】该仓位可卖出股数。T＋1的市场中sellable = 所有持仓-今日买入的仓位
        """
        return self._quantity - self._buy_today_holding_quantity - self._sell_order_quantity

    @property
    def _quantity(self):
        return self._buy_trade_quantity - self._sell_trade_quantity

    @property
    def transaction_cost(self):
        """
        【float】总费用
        """
        return self._transaction_cost

    @property
    def value_percent(self):
        """
        【float】获得该持仓的实时市场价值在总投资组合价值中所占比例，取值范围[0, 1]
        """
        accounts = ExecutionContext.accounts
        if ACCOUNT_TYPE.STOCK not in accounts:
            # FIXME 现在无法区分这个position是stock的还是benchmark的，但是benchmark因为没有用到这个字段，所以可以暂时返0处理。
            return 0
        portfolio = accounts[ACCOUNT_TYPE.STOCK].portfolio
        return 0 if portfolio.portfolio_value == 0 else self._position_value / portfolio.portfolio_value

    def _cal_close_today_amount(self, trade_amount, side):
        return 0



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .utils.logger import system_log, user_log, user_detail_log
from .events import EVENT, EventBus
from .model.commission import init_commission


class Environment(object):
    _env = None

    def __init__(self, config):
        Environment._env = self
        self._config = config
        self._data_proxy = None
        self._data_source = None
        self._event_source = None
        self._strategy_loader = None
        self._global_vars = None
        self._persist_provider = None
        self._commission_initializer = init_commission
        self.system_log = system_log
        self.user_log = user_log
        self.user_detail_log = user_detail_log
        self.event_bus = EventBus()
        self._broker = None
        self.account = None
        self.accounts = None
        self.calendar_dt = None
        self.trading_dt = None
        self.mod_dict = None
        self._universe = None
        self.profile_deco = None

    @classmethod
    def get_instance(cls):
        return Environment._env

    @property
    def config(self):
        return self._config

    def set_data_proxy(self, data_proxy):
        self._data_proxy = data_proxy

    @property
    def data_proxy(self):
        return self._data_proxy

    def set_data_source(self, data_source):
        self._data_source = data_source

    @property
    def data_source(self):
        return self._data_source

    @property
    def strategy_loader(self):
        return self._strategy_loader

    def set_strategy_loader(self, strategy_loader):
        self._strategy_loader = strategy_loader

    @property
    def global_vars(self):
        return self._global_vars

    def set_global_vars(self, global_vars):
        self._global_vars = global_vars

    def set_hold_strategy(self):
        self.config.extra.is_hold = True

    def cancel_hold_strategy(self):
        self.config.extra.is_hold = False

    @property
    def is_strategy_hold(self):
        return self.config.extra.is_hold

    def set_persist_provider(self, provider):
        self._persist_provider = provider

    @property
    def persist_provider(self):
        return self._persist_provider

    def set_event_source(self, event_source):
        self._event_source = event_source

    @property
    def event_source(self):
        return self._event_source

    @property
    def broker(self):
        return self._broker

    def set_broker(self, broker):
        self._broker = broker

    @property
    def universe(self):
        return self._universe.get()

    def update_universe(self, universe):
        self._universe.update(universe)
        self.event_bus.publish_event(EVENT.POST_UNIVERSE_CHANGED, universe)



# 可以自己import我们平台支持的第三方python模块，比如pandas、numpy等
import talib
import numpy as np


# 在这个方法中编写任何的初始化逻辑。context对象将会在你的算法策略的任何方法之间做传递
def init(context):
    # context内引入全局变量s1，存储目标合约信息
    context.s1 = "IF88"

    # 使用MACD需要设置长短均线和macd平均线的参数
    context.SHORTPERIOD = 12
    context.LONGPERIOD = 26
    context.SMOOTHPERIOD = 9
    context.OBSERVATION = 50

    # 初始化时订阅合约行情。订阅之后的合约行情会在handle_bar中进行更新
    subscribe(context.s1)


# 你选择的期货数据更新将会触发此段逻辑，例如日线或分钟线更新
def handle_bar(context, bar_dict):
    # 开始编写你的主要的算法逻辑
    # 获取历史收盘价序列，history_bars函数直接返回ndarray，方便之后的有关指标计算
    prices = history_bars(context.s1, context.OBSERVATION, '1d', 'close')

    # 用Talib计算MACD取值，得到三个时间序列数组，分别为macd,signal 和 hist
    macd, signal, hist = talib.MACD(prices, context.SHORTPERIOD, context.LONGPERIOD, context.SMOOTHPERIOD)

    # macd 是长短均线的差值，signal是macd的均线，如果短均线从下往上突破长均线，为入场信号，进行买入开仓操作
    if (macd[-1] - signal[-1] > 0 and macd[-2] - signal[-2] < 0):
        sell_qty = context.portfolio.positions[context.s1].sell_quantity
        # 先判断当前卖方仓位，如果有，则进行平仓操作
        if (sell_qty > 0):
            buy_close(context.s1, 1)
        # 买入开仓
        buy_open(context.s1, 1)

    if (macd[-1] - signal[-1] < 0 and macd[-2] - signal[-2] > 0):
        buy_qty = context.portfolio.positions[context.s1].buy_quantity
        # 先判断当前买方仓位，如果有，则进行平仓操作
        if (buy_qty > 0):
            sell_close(context.s1, 1)
        # 卖出开仓
        sell_open(context.s1, 1)


__config__ = {
    "base": {
        "strategy_type": "future",
        "start_date": "2014-09-01",
        "end_date": "2016-09-05",
        "frequency": "1d",
        "matching_type": "next_bar",
        "future_starting_cash": 1000000,
        "benchmark": None,
    },
    "extra": {
        "log_level": "error",
    },
    "mod": {
        "progress": {
            "enabled": True,
            "priority": 400,
        },
    },
    "validator": {
        "bar_limit": False,
    },
}



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from six import itervalues

from ...const import DAYS_CNT
from ...utils.repr import property_repr


class BasePortfolio(object):

    __repr__ = property_repr

    def __init__(self, cash, start_date, account_type):
        self._account_type = account_type
        self._yesterday_portfolio_value = cash
        self._yesterday_units = cash
        self._cash = cash
        self._starting_cash = cash
        self._units = cash
        self._start_date = start_date
        self._current_date = start_date

        self._frozen_cash = 0.
        self._total_commission = 0.
        self._total_tax = 0.

        self._dividend_receivable = 0.
        self._dividend_info = {}

    @property
    def _type(self):
        return self._account_type

    @property
    def _pid(self):
        return self._account_type.value

    @property
    def daily_returns(self):
        """
        【float】当前最新一天的每日收益
        """
        return 0 if self.yesterday_unit_net_value == 0 else self.unit_net_value / self.yesterday_unit_net_value - 1

    @property
    def yesterday_unit_net_value(self):
        return 0 if self._yesterday_units == 0 else self._yesterday_portfolio_value / self._yesterday_units

    @property
    def unit_net_value(self):
        return 0 if self._units == 0 else self.portfolio_value / self._units

    @property
    def yesterday_units(self):
        return self._yesterday_units

    @property
    def units(self):
        return self._units

    @property
    def starting_cash(self):
        """
        【float】回测或实盘交易给算法策略设置的初始资金
        """
        return self._starting_cash

    @property
    def start_date(self):
        """
        【datetime.datetime】策略投资组合的回测/实时模拟交易的开始日期
        """
        return self._start_date

    @property
    def frozen_cash(self):
        """
        【float】冻结资金
        """
        return self._frozen_cash

    @property
    def cash(self):
        # 可用资金
        raise NotImplementedError

    @property
    def portfolio_value(self):
        """
        【float】总权益，包含市场价值和剩余现金
        """
        # 投资组合总值
        raise NotImplementedError

    @property
    def positions(self):
        # 持仓
        raise NotImplementedError

    @property
    def daily_pnl(self):
        """
        【float】当日盈亏，当日投资组合总权益-昨日投资组合总权益
        """
        # 当日盈亏
        raise NotImplementedError

    @property
    def market_value(self):
        """
        【float】投资组合当前所有证券仓位的市值的加总
        """
        return sum(position.market_value for position in itervalues(self.positions))

    @property
    def pnl(self):
        """
        【float】当前投资组合的累计盈亏
        """
        # 总盈亏
        return self.portfolio_value - self.starting_cash

    @property
    def total_returns(self):
        """
        【float】投资组合至今的累积收益率。计算方法是现在的投资组合价值/投资组合的初始资金
        """
        # 总收益率
        return 0 if self.starting_cash == 0 else self.pnl / self.starting_cash

    @property
    def annualized_returns(self):
        """
        【float】投资组合的年化收益率
        """
        # 年化收益率
        return (1 + self.total_returns) ** (
            DAYS_CNT.DAYS_A_YEAR / float((self._current_date - self.start_date).days + 1)) - 1

    @property
    def transaction_cost(self):
        """
        【float】总费用
        """
        # 总费用
        return self._total_commission + self._total_tax



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six

from .base_portfolio import BasePortfolio
from ..dividend import Dividend
from ..position import Positions, PositionsClone, StockPosition
from ...utils.repr import dict_repr


StockPersistMap = {
    "_yesterday_portfolio_value": "_yesterday_portfolio_value",
    "_yesterday_units": "_yesterday_units",
    "_cash": "_cash",
    "_starting_cash": "_starting_cash",
    "_units": "_units",
    "_start_date": "_start_date",
    "_current_date": "_current_date",
    "_frozen_cash": "_frozen_cash",
    "_total_commission": "_total_commission",
    "_total_tax": "_total_tax",
    "_dividend_receivable": "_dividend_receivable",
    "_dividend_info": "_dividend_info",
    "_positions": "_positions",
}


# ET：如果这里修改的话，记得提醒我修改pickle那里，多谢
class StockPortfolioClone(object):
    __repr__ = dict_repr


class StockPortfolio(BasePortfolio):

    def __init__(self, cash, start_date, account_type):
        super(StockPortfolio, self).__init__(cash, start_date, account_type)
        self._positions = Positions(StockPosition)
        self._portfolio_value = None

    def restore_from_dict_(self, portfolio_dict):
        self._cash = portfolio_dict['_cash']
        self._start_date = portfolio_dict['_start_date']
        self._positions.clear()
        self._dividend_info.clear()
        for persist_key, origin_key in six.iteritems(StockPersistMap):
            if persist_key == "_dividend_info":
                tmp = {}
                for order_book_id, dividend_dict in six.iteritems(portfolio_dict[persist_key]):
                    tmp[order_book_id] = Dividend.__from_dict__(dividend_dict)
                setattr(self, origin_key, tmp)
            elif persist_key == "_positions":
                for order_book_id, position_dict in six.iteritems(portfolio_dict[persist_key]):
                    self._positions[order_book_id] = StockPosition.__from_dict__(position_dict)
            else:
                try:
                    setattr(self, origin_key, portfolio_dict[persist_key])
                except KeyError as e:
                    if persist_key in ["_yesterday_units", "_units"]:
                        # FIXME 对于已有 persist_key 做暂时error handling 处理。
                        setattr(self, origin_key, portfolio_dict["_starting_cash"])
                    else:
                        raise e

    def __to_dict__(self):
        p_dict = {}
        for persist_key, origin_key in six.iteritems(StockPersistMap):
            if persist_key == "_dividend_info":
                p_dict[persist_key] = {oid: dividend.__to_dict__() for oid, dividend in six.iteritems(getattr(self, origin_key))}
            elif persist_key == "_positions":
                p_dict[persist_key] = {oid: position.__to_dict__() for oid, position in six.iteritems(getattr(self, origin_key))}
            else:
                p_dict[persist_key] = getattr(self, origin_key)
        return p_dict

    @property
    def cash(self):
        """
        【float】可用资金
        """
        return self._cash

    @property
    def positions(self):
        """
        【dict】一个包含股票子组合仓位的字典，以order_book_id作为键，position对象作为值，关于position的更多的信息可以在下面的部分找到。
        """
        return self._positions

    @property
    def daily_pnl(self):
        """
        【float】当日盈亏，当日投资组合总权益-昨日投资组合总权益
        """
        return self.portfolio_value - self._yesterday_portfolio_value

    @property
    def portfolio_value(self):
        """
        【float】总权益，包含市场价值和剩余现金
        """
        if self._portfolio_value is None:
            # 总资金 + Sum(position._position_value)
            self._portfolio_value = self.cash + self.frozen_cash + sum(
                position._position_value for position in six.itervalues(self.positions))

        return self._portfolio_value

    @property
    def dividend_receivable(self):
        """
        【float】投资组合在分红现金收到账面之前的应收分红部分。具体细节在分红部分
        """
        return self._dividend_receivable

    def _clone(self):
        p = StockPortfolioClone()
        for key in dir(self):
            if "__" in key:
                continue
            if key == "positions":
                ps = PositionsClone(StockPosition)
                for order_book_id in self.positions:
                    ps[order_book_id] = self.positions[order_book_id]._clone()
                setattr(p, key, ps)
            else:
                setattr(p, key, getattr(self, key))
        return p



# -*- coding: utf-8 -*-
#
# Copyright 2017 Ricequant, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six

from ..margin import Margin
from ...const import SIDE, POSITION_EFFECT, ACCOUNT_TYPE
from ...utils.i18n import gettext as _
from ...utils.logger import user_system_log
from ...execution_context import ExecutionContext

from .base_account import BaseAccount


class FutureAccount(BaseAccount):
    def __init__(self, env, init_cash, start_date):
        super(FutureAccount, self).__init__(env, init_cash, start_date, ACCOUNT_TYPE.FUTURE)
        self._margin_decider = Margin(env.config.base.margin_multiplier)

    @property
    def margin_decider(self):
        return self._margin_decider

    def before_trading(self):
        super(FutureAccount, self).before_trading()
        positions = self.portfolio.positions
        removing_ids = []
        for order_book_id in positions.keys():
            position = positions[order_book_id]
            if position._quantity == 0 and position._buy_open_order_quantity == 0 \
                    and position._sell_open_order_quantity == 0:
                removing_ids.append(order_book_id)
        for order_book_id in removing_ids:
            positions.pop(order_book_id, None)

    def after_trading(self):
        pass

    def settlement(self):
        portfolio = self.portfolio
        portfolio._portfolio_value = None
        positions = portfolio.positions
        data_proxy = ExecutionContext.get_data_proxy()
        trading_date = ExecutionContext.get_current_trading_dt().date()

        for order_book_id, position in six.iteritems(positions):
            settle_price = data_proxy.get_settle_price(order_book_id, trading_date)
            position._last_price = settle_price
            self._update_market_value(position, settle_price)

        self.portfolio_persist()

        portfolio._yesterday_portfolio_value = portfolio.portfolio_value

        de_listed_id_list = []
        for order_book_id, position in six.iteritems(positions):
            # 检查合约是否到期,如果到期,则按照结算价来进行平仓操作
            if position._de_listed_date is not None and trading_date >= position._de_listed_date.date():
                de_listed_id_list.append(order_book_id)
            else:
                settle_price = data_proxy.get_settle_price(order_book_id, trading_date)
                self._update_holding_by_settle(position, settle_price)
                position._daily_realized_pnl = 0
                position._buy_daily_realized_pnl = 0
                position._sell_daily_realized_pnl = 0
        for de_listed_id in de_listed_id_list:
            if positions[de_listed_id]._quantity != 0:
                user_system_log.warn(
                    _("{order_book_id} is expired, close all positions by system").format(order_book_id=de_listed_id))
            del positions[de_listed_id]

        portfolio._daily_transaction_cost = 0

    def bar(self, bar_dict):
        portfolio = self.portfolio
        portfolio._portfolio_value = None
        positions = portfolio.positions

        for order_book_id, position in six.iteritems(positions):
            bar = bar_dict[order_book_id]
            if not bar.isnan:
                position._last_price = bar.close
                self._update_market_value(position, bar.close)

    def tick(self, tick):
        portfolio = self.portfolio
        portfolio._portfolio_value = None
        position = portfolio.positions[tick.order_book_id]
        position._last_price = tick.last
        self._update_market_value(position, tick.last)

    def order_pending_new(self, account, order):
        if self != account:
            return
        if order._is_final():
            return
        order_book_id = order.order_book_id
        position = self.portfolio.positions[order_book_id]
        position._total_orders += 1
        created_quantity = order.quantity
        created_value = order._frozen_price * created_quantity * position._contract_multiplier
        frozen_margin = self.margin_decider.cal_margin(order_book_id, order.side, created_value)
        self._update_order_data(position, order, created_quantity, created_value)
        self._update_frozen_cash(order, frozen_margin)

    def on_order_creation_pass(self, account, order):
        pass

    def order_creation_reject(self, account, order):
        if self != account:
            return
        order_book_id = order.order_book_id
        position = self.portfolio.positions[order_book_id]
        position._total_orders -= 1
        cancel_quantity = order.unfilled_quantity
        cancel_value = -order._frozen_price * cancel_quantity * position._contract_multiplier
        frozen_margin = self.margin_decider.cal_margin(order_book_id, order.side, cancel_value)
        self._update_order_data(position, order, -cancel_quantity, cancel_value)
        self._update_frozen_cash(order, frozen_margin)

    def order_pending_cancel(self, account, order):
        pass

    def order_cancellation_pass(self, account, order):
        self._cancel_order_cal(account, order)

    def order_cancellation_reject(self, account, order):
        pass

    def order_unsolicited_update(self, account, order):
        self._cancel_order_cal(account, order)

    def _cancel_order_cal(self, account, order):
        if self != account:
            return
        order_book_id = order.order_book_id
        position = self.portfolio.positions[order.order_book_id]
        rejected_quantity = order.unfilled_quantity
        rejected_value = -order._frozen_price * rejected_quantity * position._contract_multiplier
        frozen_margin = self.margin_decider.cal_margin(order_book_id, order.side, rejected_value)
        self._update_order_data(position, order, -rejected_quantity, rejected_value)
        self._update_frozen_cash(order, frozen_margin)

    def trade(self, account, trade):
        if self != account:
            return
        order = trade.order
        order_book_id = order.order_book_id
        bar_dict = ExecutionContext.get_current_bar_dict()
        portfolio = self.portfolio
        portfolio._portfolio_value = None
        position = portfolio.positions[order_book_id]
        position._is_traded = True
        position._total_trades += 1
        trade_quantity = trade.last_quantity

        if order.position_effect == POSITION_EFFECT.OPEN:
            if order.side == SIDE.BUY:
                position._buy_avg_open_price = (position._buy_avg_open_price * position.buy_quantity + trade_quantity *
                                                trade.last_price) / (position.buy_quantity + trade_quantity)
            elif order.side == SIDE.SELL:
                position._sell_avg_open_price = (position._sell_avg_open_price * position.sell_quantity +
                                                 trade_quantity * trade.last_price) / (position.sell_quantity +
                                                                                  trade_quantity)

        minus_value_by_trade = -order._frozen_price * trade_quantity * position._contract_multiplier
        trade_value = trade.last_price * trade_quantity * position._contract_multiplier
        frozen_margin = self.margin_decider.cal_margin(order_book_id, order.side, minus_value_by_trade)

        portfolio._total_tax += trade.tax
        portfolio._total_commission += trade.commission
        portfolio._daily_transaction_cost = portfolio._daily_transaction_cost + trade.tax + trade.commission

        self._update_frozen_cash(order, frozen_margin)
        self._update_order_data(position, order, -trade_quantity, minus_value_by_trade)
        self._update_trade_data(position, trade, trade_quantity, trade_value)

        self._update_market_value(position, bar_dict[order_book_id].close)
        self._last_trade_id = trade.exec_id

    @staticmethod
    def _update_holding_by_settle(position, settle_price):
        position._prev_settle_price = settle_price
        position._buy_old_holding_list += position._buy_today_holding_list
        position._sell_old_holding_list += position._sell_today_holding_list
        position._buy_today_holding_list = []
        position._sell_today_holding_list = []

    def _update_trade_data(self, position, trade, trade_quantity, trade_value):
        """
        计算 [buy|sell]_trade_[value|quantity]
        计算 [buy|sell]_[open|close]_trade_quantity
        计算 [buy|sell]_settle_holding
        计算 [buy|sell]_today_holding_list
        计算 [buy|sell]_holding_cost
        计算 [bar|sell]_margin
        计算 daily_realized_pnl
        """
        order = trade.order

        if order.side == SIDE.BUY:
            if order.position_effect == POSITION_EFFECT.OPEN:
                position._buy_open_trade_quantity += trade_quantity
                position._buy_open_trade_value += trade_value
                position._buy_open_transaction_cost += trade.commission
                position._buy_today_holding_list.insert(0, (trade.last_price, trade_quantity))
            else:
                position._buy_close_trade_quantity += trade_quantity
                position._buy_close_trade_value += trade_value
                position._buy_close_transaction_cost += trade.commission
                delta_daily_realized_pnl = self._update_holding_by_close_action(trade)
                position._daily_realized_pnl += delta_daily_realized_pnl
                position._sell_daily_realized_pnl += delta_daily_realized_pnl
            position._buy_trade_quantity += trade_quantity
            position._buy_trade_value += trade_value
        else:
            if order.position_effect == POSITION_EFFECT.OPEN:
                position._sell_open_trade_quantity += trade_quantity
                position._sell_open_trade_value += trade_value
                position._sell_open_transaction_cost += trade.commission
                position._sell_today_holding_list.insert(0, (trade.last_price, trade_quantity))
            else:
                position._sell_close_trade_quantity += trade_quantity
                position._sell_close_trade_value += trade_value
                position._sell_close_transaction_cost += trade.commission
                delta_daily_realized_pnl = self._update_holding_by_close_action(trade)
                position._daily_realized_pnl += delta_daily_realized_pnl
                position._buy_daily_realized_pnl += delta_daily_realized_pnl
            position._sell_trade_quantity += trade_quantity
            position._sell_trade_value += trade_value

    @staticmethod
    def _update_order_data(position, order, inc_order_quantity, inc_order_value):
        if order.side == SIDE.BUY:
            if order.position_effect == POSITION_EFFECT.OPEN:
                position._buy_open_order_quantity += inc_order_quantity
                position._buy_open_order_value += inc_order_value
            else:
                position._buy_close_order_quantity += inc_order_quantity
                position._buy_close_order_value += inc_order_value
        else:
            if order.position_effect == POSITION_EFFECT.OPEN:
                position._sell_open_order_quantity += inc_order_quantity
                position._sell_open_order_value += inc_order_value
            else:
                position._sell_close_order_quantity += inc_order_quantity
                position._sell_close_order_value += inc_order_value

    def _update_frozen_cash(self, order, inc_order_value):
        if order.position_effect == POSITION_EFFECT.OPEN:
            self.portfolio._frozen_cash += inc_order_value

    def _update_holding_by_close_action(self, trade):
        order = trade.order
        order_book_id = order.order_book_id
        position = self.portfolio.positions[order_book_id]
        settle_price = position._prev_settle_price
        left_quantity = trade.last_quantity
        delta_daily_realized_pnl = 0
        if order.side == SIDE.BUY:
            # 先平昨仓
            while True:
                if left_quantity == 0:
                    break
                if len(position._sell_old_holding_list) == 0:
                    break
                oldest_price, oldest_quantity = position._sell_old_holding_list.pop()
                if oldest_quantity > left_quantity:
                    consumed_quantity = left_quantity
                    position._sell_old_holding_list.append((oldest_price, oldest_quantity - left_quantity))
                else:
                    consumed_quantity = oldest_quantity
                left_quantity -= consumed_quantity
                delta_daily_realized_pnl += self._cal_daily_realized_pnl(trade, settle_price, consumed_quantity)
            # 再平今仓
            while True:
                if left_quantity <= 0:
                    break
                oldest_price, oldest_quantity = position._sell_today_holding_list.pop()
                if oldest_quantity > left_quantity:
                    consumed_quantity = left_quantity
                    position._sell_today_holding_list.append((oldest_price, oldest_quantity - left_quantity))
                else:
                    consumed_quantity = oldest_quantity
                left_quantity -= consumed_quantity
                delta_daily_realized_pnl += self._cal_daily_realized_pnl(trade, oldest_price, consumed_quantity)
        else:
            # 先平昨仓
            while True:
                if left_quantity == 0:
                    break
                if len(position._buy_old_holding_list) == 0:
                    break
                oldest_price, oldest_quantity = position._buy_old_holding_list.pop()
                if oldest_quantity > left_quantity:
                    consumed_quantity = left_quantity
                    position._buy_old_holding_list.append((oldest_price, oldest_quantity - left_quantity))
                else:
                    consumed_quantity = oldest_quantity
                left_quantity -= consumed_quantity
                delta_daily_realized_pnl += self._cal_daily_realized_pnl(trade, settle_price, consumed_quantity)
            # 再平今仓
            while True:
                if left_quantity <= 0:
                    break
                oldest_price, oldest_quantity = position._buy_today_holding_list.pop()
                if oldest_quantity > left_quantity:
                    consumed_quantity = left_quantity
                    position._buy_today_holding_list.append((oldest_price, oldest_quantity - left_quantity))
                    left_quantity = 0
                else:
                    consumed_quantity = oldest_quantity
                left_quantity -= consumed_quantity
                delta_daily_realized_pnl += self._cal_daily_realized_pnl(trade, oldest_price, consumed_quantity)
        return delta_daily_realized_pnl

    @staticmethod
    def _update_market_value(position, price):
        """
        计算 market_value
        计算 pnl
        计算 daily_holding_pnl
        """
        botq = position._buy_open_trade_quantity
        sctq = position._sell_close_trade_quantity
        bctq = position._buy_close_trade_quantity
        sotq = position._sell_open_trade_quantity
        position._buy_market_value = (botq - sctq) * price * position._contract_multiplier
        position._sell_market_value = (bctq - sotq) * price * position._contract_multiplier
        position._market_value = position._buy_market_value + position._sell_market_value

    def _cal_daily_realized_pnl(self, trade, cost_price, consumed_quantity):
        order = trade.order
        position = self.portfolio.positions[order.order_book_id]
        if order.side == SIDE.BUY:
            return (cost_price - trade.last_price) * consumed_quantity * position._contract_multiplier
        else:
            return (trade.last_price - cost_price) * consumed_quantity * position._contract_multiplier
